from __future__ import print_function, division

import os
import wx

from monty.os.path import which
from monty.string import is_string


__all__ = [
    "showErrorMessage",
    "askUser",
    "showLicense",
]


# Helper functions.
def _straceback():
    """Returns a string with the traceback."""
    import traceback
    return traceback.format_exc()


def askUser(parent, message):
    """Open dialog with message, return user's answer."""
    ask = wx.MessageDialog(parent, message)
    answer = ask.ShowModal() == wx.ID_OK
    ask.Destroy()
    return answer


class ErrorDialog(wx.MessageDialog):
    def __init__(self, parent, message):
        super(ErrorDialog, self).__init__(parent, message=message, caption='Error Message',
                                          style=wx.YES_NO | wx.CANCEL | wx.NO_DEFAULT | wx.ICON_ERROR | wx.STAY_ON_TOP)

def showErrorMessage(parent, message=None):
    """
    Open a `MessageDialog` with an error message.
    If message is None, the python traceback is used.
    """
    if message is None: message = _straceback()

    message += "\n\n Do you want to send a bug report?"
    dialog = ErrorDialog(parent, message)

    # Send mail if the user clicked YES.
    if dialog.ShowModal() == wx.ID_YES:
        mail = SendMail(parent)
        mail.setSender(_user_at_host())
        mail.setSubject("Bug report")
        mail.setBody(message)
        mail.ShowModal()
        mail.Destroy()

    dialog.Destroy()


def showLicense(parent=None, codename=None):
    codename = "Abipy" if codename is None else codename

    license_text = """%(codename)s is free software; you can redistribute
it and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the License,
or (at your option) any later version.

%(codename)s is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details. You should have
received a copy of the GNU General Public License along with the code;
if not, write to the Free Software Foundation, Inc., 59 Temple Place,
Suite 330, Boston, MA  02111-1307  USA""" % {"codename": codename}

    dialog = License(parent, license_text)
    dialog.ShowModal()
    dialog.Destroy()


class License(wx.Dialog):
    def __init__(self, parent, license_text,  **kwargs):
        wx.Dialog.__init__ (self, parent, id=-1, title="License")

        vsizer = wx.BoxSizer( wx.VERTICAL )

        text = wx.TextCtrl( self, -1, license_text, style=wx.TE_MULTILINE | wx.TE_READONLY )
        vsizer.Add(text, 0, wx.ALL|wx.EXPAND, 5 )

        vsizer.Add(wx.Button( self, wx.ID_OK), 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 5 )

        self.SetSizerAndFit(vsizer)


class SendMail(wx.Dialog):
    """This dialog allows the user to send an email with sendmail."""
    def __init__(self, parent, title="SendMail"):
        super(SendMail, self).__init__(parent, -1, title, wx.DefaultPosition, wx.Size(400, 420))

        panel = wx.Panel(self, -1)
        vbox = wx.BoxSizer(wx.VERTICAL)
        hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        hbox3 = wx.BoxSizer(wx.HORIZONTAL)
        st1 = wx.StaticText(panel, -1, 'From: ')
        st2 = wx.StaticText(panel, -1, 'To: ')
        st3 = wx.StaticText(panel, -1, 'Subject: ')
        self.sender = wx.TextCtrl(panel, -1, size=(180, -1))
        self.mailto = wx.TextCtrl(panel, -1, size=(180, -1))
        self.subject = wx.TextCtrl(panel, -1, size=(180, -1))
        self.body = wx.TextCtrl(panel, -1, style=wx.TE_MULTILINE)
        button_send = wx.Button(panel, 1, 'Send')
        hbox1.Add(st1, 0, wx.LEFT, 10)
        hbox1.Add(self.sender, 0, wx.LEFT, 20)
        hbox2.Add(st2, 0, wx.LEFT, 10)
        hbox2.Add(self.mailto, 0, wx.LEFT, 35)
        hbox3.Add(st3, 0, wx.LEFT, 10)
        hbox3.Add(self.subject, 0)
        vbox.Add(hbox1, 0, wx.TOP, 10)
        vbox.Add(hbox2, 0, wx.TOP, 10)
        vbox.Add(hbox3, 0, wx.TOP, 10)
        vbox.Add(self.body, 1, wx.EXPAND | wx.TOP | wx.RIGHT | wx.LEFT, 15)
        vbox.Add(button_send, 0, wx.ALIGN_CENTER | wx.TOP | wx.BOTTOM, 20)
        self.Bind(wx.EVT_BUTTON, self.onSend, id=1)
        panel.SetSizer(vbox)
        self.Centre()

    def setSender(self, s):
        self.sender.SetValue(s)

    def setMailto(self, s):
        self.mailto.SetValue(s)

    def setSubject(self, s):
        self.subject.SetValue(s)

    def setBody(self, text):
        self.body.SetValue(text)

    def onSend(self, event):
        sender = self.sender.GetValue()
        mailto = self.mailto.GetValue()
        subject = self.subject.GetValue()
        body = self.body.GetValue()

        header = 'From: %s\r\nTo: %s\r\nSubject: %s\r\n\r\n' % (sender, mailto, subject)
        
        try:
            retcode = sendmail(subject, body, mailto, sender=sender)

            if retcode:
                dialog = wx.MessageDialog(self, 'Email was not sent', 'Failure', wx.OK | wx.ICON_INFORMATION)
            else:
                dialog = wx.MessageDialog(self, 'Email was successfully sent', 'Success', wx.OK | wx.ICON_INFORMATION)

            dialog.ShowModal()
            dialog.Destroy()

        except:
            showErrorMessage(self)


def _user_at_host():
    from socket import gethostname
    return os.getlogin() + "@" + gethostname()


def sendmail(subject, text, mailto, sender=None):
    """
    Sends an e-mail with unix sendmail. 

    Args:
        subject:
            String with the subject of the mail.
        text:
            String with the body of the mail.
        mailto:
            String or list of string with the recipients.
        sender:
            string with the sender address.
            If sender is None, username@hostname is used.

    Returns:
        exit status
    """
    # Body of the message.
    sender = _user_at_host() if sender is None else sender
    if is_string(mailto): mailto = [mailto]

    from email.mime.text import MIMEText

    mail = MIMEText(text)
    mail["Subject"] = subject
    mail["From"] = sender
    mail["To"] = ", ".join(mailto)

    msg = mail.as_string()

    # sendmail works much better than the python interface.
    # Note that sendmail is available only on Unix-like OS.
    from subprocess import Popen, PIPE

    sendmail = which("sendmail")
    if sendmail is None: return -1

    p = Popen([sendmail, "-t"], stdin=PIPE, stderr=PIPE)

    outdata, errdata = p.communicate(msg)
    return len(errdata)


class MyApp(wx.App):
    def OnInit(self):
        dialog = SendMail(None)
        dialog.ShowModal()
        dialog.Destroy()
        return True


if __name__  == "__main__":
    MyApp(0).MainLoop()



"""Collections of Popup menus associated to filenames."""
import wx
import wx.lib.dialogs as wxdg
import abipy.gui.awx as awx
import abipy.gui.electronswx as ewx

from collections import OrderedDict
from abipy.abilab import abifile_subclass_from_filename, abiopen
from abipy.core.mixins import NcDumper, get_filestat
from abipy.abio.outputs import AbinitLogFile, AbinitOutputFile
from abipy.iotools.visualizer import Visualizer
from abipy.waves import WfkFile
from abipy.electrons import SigresFile, GsrFile
from abipy.electrons.bse import MdfFile
from abipy.gui.events import AbinitEventsFrame
from abipy.gui.timer import AbinitTimerFrame
from abipy.gui.editor import SimpleTextViewer, MyEditorFrame
from abipy.gui.wxncview import NcViewerFrame


__all__ = [
    "popupmenu_for_filename",
]


def popupmenu_for_filename(parent, filename):
    """
    Factory function that returns the appropriate popup menu.

    Args:
        parent:
            The parent wx window. Used to connect the children created
            by the popupmenu to the calling window.
    """
    menu = PopupMenu.from_filename(filename)

    if menu is not None:
        menu.set_target(filename)
        menu.set_parent(parent)

    return menu

#--------------------------------------------------------------------------------------------------
# Callbacks


def showNcdumpMessage(parent, filepath):
    """Open a dialog with the output of ncdump."""
    title = "ncdump output for file %s" % filepath
    text = NcDumper().dump(filepath)
    # TODO: Get a decent wxpython editor somewhere
    #SimpleTextViewer(parent, text, title=title).Show()
    MyEditorFrame.from_text(parent, text, title=title).Show()

def openWxncview(parent, filepath):
    """Open a dialog with the output of ncdump."""
    title = "wxncview:  %s" % filepath
    text = NcDumper().dump(filepath)
    NcViewerFrame(parent, filepath, title=title).Show()


def showFileStat(parent, filepath):
    """Open a dialog reporting file stats."""
    caption = "Info on file %s" % filepath
    stat_dict = get_filestat(filepath)
    msg = str(stat_dict)
    style = wx.DEFAULT_FRAME_STYLE
    wxdg.ScrolledMessageDialog(parent, msg, caption=caption, size=(600, 600), style=style).Show()


def showAbinitEventsFrame(parent, filepath):
    """Open a dialog reporting file stats."""
    AbinitEventsFrame(parent, filepath).Show()


def showAbinitTimerFrame(parent, filepath):
    """Open a dialog reporting file stats."""
    try:
        frame = AbinitTimerFrame(parent, filepath)
        frame.Show()
    except awx.Error as exc:
        awx.showErrorMessage(parent, str(exc))


def showStructure(parent, filepath):
    ncfile = abiopen(filepath)
    visu_classes = Visualizer.get_available(ext="xsf")
    if not visu_classes:
        print("Not visualizer found for extension xsf")
        return
    vname = visu_classes[0].name

    visu = ncfile.structure.visualize(vname)

    thread = awx.WorkerThread(parent, target=visu)
    thread.start()



class PopupMenu(wx.Menu):
    """
    Base class for popup menus. `A PopupMenu` has a list of callback functions
    indexed by the menu title. The signature of the callback function is func(parent, filename) where
    filename is the name of the file selected in the Widget and parent is the wx
    Window that will become the parent of the new frame created by the callback.
    """
    MENU_TITLES = OrderedDict([
    ])

    HANDLED_FILES = []

    def __init__(self):
        super(PopupMenu, self).__init__()
        self._make_menu()

    @staticmethod
    def from_filename(filename):
        """
        Static factory function that instanciates the appropriate subclass of `NcFilePopupMenu`
        Returns None if the extesion of filename is not supported.
        """
        # Find the AbinitNcFile subclass associated to files.
        try:
            file_class = abifile_subclass_from_filename(filename)
        except KeyError:
            if filename.endswith(".nc"): NcFilePopupMenu()
            return None

        # Check whether a subclass handles this file..
        # Fallback to a simple PopupMenu if no match.
        def allsubclasses(cls):
            """Returns the set of subclasses of cls."""
            children = [cls]
            for sc in cls.__subclasses__():
                if sc.__subclasses__():
                    for k in sc.__subclasses__():
                        children.extend(allsubclasses(k))
                else:
                    children.append(sc)
            return set(children)

        for cls in allsubclasses(PopupMenu):
            if cls.handle_file_class(file_class):
                return cls()
        else:
            if filename.endswith(".nc"): NcFilePopupMenu()
            return PopupMenu()

    @classmethod
    def handle_file_class(cls, file_class):
        """True if the popupmenu is associated to file_class."""
        return file_class in cls.HANDLED_FILES

    def _make_menu(self):
        """Build the menu taking into account the options of the superclasses."""
        base_classes = list(self.__class__.__bases__) + [self.__class__]
        base_classes.reverse()

        assert not hasattr(self, "menu_title_by_id")
        assert not hasattr(self, "menu_titles")
        self.menu_title_by_id, self.menu_titles = OrderedDict(), OrderedDict()

        for cls in base_classes:
            try:
                menus = cls.MENU_TITLES
            except AttributeError as exc:
                awx.WARNING("exc ",exc," for cls", cls)
                continue

            self.menu_titles.update(menus)

            for title in menus:
                self.menu_title_by_id[wx.NewId()] = title

            # Add sentinel for Menu separator.
            self.menu_title_by_id["separator_" + str(len(self.menu_titles))] = None

        for (id, title) in self.menu_title_by_id.items():
            if title is None:
                sepid = int(id.split("_")[-1])
                if sepid != len(self.menu_titles):
                    self.AppendSeparator()
            else:
                # Register menu handlers with EVT_MENU, on the menu.
                self.Append(id, title)
                wx.EVT_MENU(self, id, self.OnMenuSelection)

    def set_parent(self, parent):
        """Set the parent window."""
        self._parent = parent

    @property
    def parent(self):
        """Returns the parent window."""
        try:
            return self._parent
        except AttributeError:
            awx.WARNING("Popup menu doesn't have parent")
            return None

    def set_target(self, target):
        """Set the target of the callback."""
        self._target = target

    @property
    def target(self):
        """The target of the callback."""
        try:
            return self._target
        except AttributeError:
            return None

    def _get_callback(self, title):
        return self.menu_titles[title]

    def OnMenuSelection(self, event):
        title = self.menu_title_by_id[event.GetId()]
        callback = self._get_callback(title)
        #print("Calling callback %s on target %s" % (callback, self.target))
        try:
            callback(parent=self.parent, filepath=self.target)
        except:
            awx.showErrorMessage(parent=self.parent)


class AbinitTextFilePopupMenu(PopupMenu):
    """
    """
    MENU_TITLES = OrderedDict([
        ("events",     showAbinitEventsFrame),
        ("properties", showFileStat),
        ("timer",      showAbinitTimerFrame),
    ])

    HANDLED_FILES = [AbinitLogFile, AbinitOutputFile]


class NcFilePopupMenu(PopupMenu):
    """
    Base class for popup menus. `A PopupMenu` has a list of callback functions
    indexed by the menu title and a list of `AbinitNcFile` associated to it.
    The signature of the callback function is func(filename, parent) where
    filename is the name of the file selected in the Widget and parent is the wx
    Window that will become the parent of the new frame created by the callback.

    How to subclass PopupMenu:

        1. Define a new class that inherits from NcFilePopupMenu.

        2. Define the callbacks in the class variable MENU_TITLES.
           Use OrderedDict to have a fixed ordering of the labels.

        3. Define the class variable HANDLED_FILES with the list of
           `AbinitNcFile` subclasses associated to the popupmenu.

        4. Done (most of the work is indeed done in the base class and in
           the factory function popupmenu_for_filename.
    """
    MENU_TITLES = OrderedDict([
        ("structure",  showStructure),
        ("ncdump",     showNcdumpMessage),
        ("wxncview",   openWxncview),
        ("properties", showFileStat),
    ])

    HANDLED_FILES = []


class EbandsPopupMenu(NcFilePopupMenu):
    """Popup menu for Netcdf files that contain the electron band structure."""
    MENU_TITLES = OrderedDict([
        ("ePlot", ewx.showElectronBandsPlot),
        ("eDos",  ewx.showElectronDosFrame),
        ("eJdos", ewx.showElectronJdosFrame),
    ])

    HANDLED_FILES = [WfkFile, GsrFile]


def showQPData(parent, filepath):
    sigres = abiopen(filepath)
    qps_spin = sigres.qplist_spin
    assert len(qps_spin) == 1
    qps_spin[0].plot_qps_vs_e0()


class SigResPopupMenu(NcFilePopupMenu):
    """Popup menu for SIGRES files."""
    MENU_TITLES = OrderedDict([
        ("qpDataPlot", showQPData),
    ])

    HANDLED_FILES = [SigresFile]


def showEXCMDF(parent, filepath):
    mdf_file = MdfFile(filepath)
    mdf_file.plot_mdfs()


class MDFPopupMenu(NcFilePopupMenu):
    """Popup menu for MDF files."""
    MENU_TITLES = OrderedDict([
        ("mdfPlot", showEXCMDF),
    ])

    HANDLED_FILES = [MdfFile]




"""Core objects and helper functions."""
from __future__ import print_function, division

import os
import sys
import wx

from monty.string import list_strings
#import abipy.tools.decorators as dec


#import logging
#logger = logging.getLogger(__name__)


__all__ = [
    "path_img",
    "makeAboutBox",
    "verbose",
    "Panel",
    "Frame",
    "get_width_height",
]

_DEBUG = True
_DEBUG = False


#if _DEBUG:
#    verbose = dec.verbose
#
#else:
def verbose(func):
    return func


#class Error(Exception):
#    """Base class for exceptions raised by awx library"""


def path_img(filename):
    """Returns the absolute path of an image."""
    dirname = os.path.dirname(__file__)
    return os.path.join(dirname, "images", filename)


def makeAboutBox(codename, version, description, developers, website=None, icon_path=None):

    licence = """%(codename)s is free software; you can redistribute 
it and/or modify it under the terms of the GNU General Public License as 
published by the Free Software Foundation; either version 2 of the License, 
or (at your option) any later version.

%(codename)s is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details. You should have 
received a copy of the GNU General Public License along with the code; 
if not, write to the Free Software Foundation, Inc., 59 Temple Place, 
Suite 330, Boston, MA  02111-1307  USA""" % {"codename": codename}

    # Make a template for the description 
    #desc = "\n".join(["\nwxPython Cookbook Chapter 5\n",
    #                  "Platform Info: (%s,%s)",
    #                  "License: Public Domain"])

    ## Get the platform information 
    #py_version = [sys.platform, ", python ", sys.version.split()[0]] 
    #platform = list(wx.PlatformInfo[1:])
    #platform[0] += (" " + wx.VERSION_STRING) 
    #wx_info = ", ".join(platform)

    #info.SetDescription(desc % (py_version, wx_info))

    info = wx.AboutDialogInfo()

    if icon_path is not None:
        info.SetIcon(wx.Icon(icon_path, wx.BITMAP_TYPE_PNG))

    info.SetName(codename)
    info.SetVersion(version)
    info.SetDescription(description)
    info.SetCopyright('(C) Abipy group')

    if website is not None:
        info.SetWebSite(website)

    info.SetLicence(licence)

    for dev in list_strings(developers):
        info.AddDeveloper(dev)

    wx.AboutBox(info)


def get_width_height(window, string, pads=(10, 10)):
    """
    Returns the width and the height (in pixels) of a string used in window.
    Returns are padded with pads

    See http://docs.wxwidgets.org/2.8/wx_wxdc.html#wxdcgettextextent
    """
    f = window.GetFont()
    dc = wx.WindowDC(window)
    dc.SetFont(f)
    width, height = dc.GetTextExtent(string)
    return width + pads[0], height + pads[1]


class MyWindow(object):
    """
    Mixin class providing helper functions and commonly used callbacks.

    Attributes:
        HELP_MSG:
            string with a short help (will be displayed in MessageDialog in onHelp callback)
    """
    HELP_MSG = "No help available"

    def getParentWithType(self, cls):
        """
        Returns the first parent window of type cls.

        Raises:
            RuntimeError if we have reached the head of the linked list.
        """
        parent = self.GetParent() 

        while True: 
            if parent is None:
                raise RuntimeError("Cannot find parent with class %s, reached None parent!" % cls)

            if isinstance(parent, cls): 
                return parent
            else:
                parent = parent.GetParent()

    def onHelp(self, event):
        """Short help."""
        dialog = wx.MessageDialog(self, self.HELP_MSG, " Quick Reference", wx.OK | wx.ICON_INFORMATION)
        dialog.ShowModal()
        dialog.Destroy()


class Panel(wx.Panel, MyWindow):
    def __init__(self, parent, *args, **kwargs):
        super(Panel, self).__init__(parent, *args, **kwargs)


class Frame(wx.Frame, MyWindow):
    """Base class for frames."""

    def __init__(self, parent, *args, **kwargs):
        if "title" not in kwargs:
            kwargs["title"] = self.__class__.__name__
        
        super(Frame, self).__init__(parent, *args, **kwargs)



"""Panels to interact with the SIGEPH file."""
import param
import panel as pn
import panel.widgets as pnw
import bokeh.models.widgets as bkw

from .core import PanelWithElectronBands #, PanelWithEbandsRobot


class SigEPhFilePanel(PanelWithElectronBands):
    """
    Panel with widgets to interact with a |SigEphFile|.
    """

    plot_plot_qpsolution_btn = pnw.Button(name="Plot Sigma_nk", button_type='primary')

    def __init__(self, sigeph, **params):
        super().__init__(**params)
        self.sigeph = sigeph

        self.sigma_spin_select = pn.widgets.Select(name="Spin index", options=list(range(sigeph.nsppol)))
        self.sigma_kpoint_select = pn.widgets.Select(name="Kpoint in Sigma_nk",
                options={"[%d]: %s" % (ik, repr(k)): ik for ik, k in enumerate(sigeph.sigma_kpoints)})

        #sigma_band_select  = param.ObjectSelector(default=0, objects=[0], doc="Band index in sigma_nk")

    @property
    def ebands(self):
        """|ElectronBands|."""
        return self.sigeph.ebands

    def plot_lws(self):
        # Insert results in grid.
        gspec = pn.GridSpec(sizing_mode='scale_width')
        for i, rta_type in enumerate(("serta", "mrta")):
            fig = self.sigeph.plot_lws_vs_e0(rta_type=rta_type, **self.fig_kwargs)
            gspec[i, 0] = fig
            fig = self.sigeph.plot_tau_vtau(rta_type=rta_type, **self.fig_kwargs)
            gspec[i, 1] = fig

        return gspec

    def plot_qpgaps(self):
        # Insert results in grid.
        gspec = pn.GridSpec(sizing_mode='scale_width')

        for i, qp_kpt in enumerate(self.sigeph.sigma_kpoints):
            fig = self.sigeph.plot_qpgaps_t(qp_kpoints=qp_kpt, qp_type="qpz0", **self.fig_kwargs)
            gspec[i, 0] = fig
            fig = self.sigeph.plot_qpgaps_t(qp_kpoints=qp_kpt, qp_type="otms", **self.fig_kwargs)
            gspec[i, 1] = fig

        return gspec

    def plot_qps_vs_e0(self):
        return self._mp(self.sigeph.plot_qps_vs_e0(**self.fig_kwargs))

    @param.depends('plot_plot_qpsolution_btn.clicks')
    def plot_qpsolution_sk(self):
        if self.plot_plot_qpsolution_btn.clicks == 0: return
        fig = self.sigeph.plot_qpsolution_sk(self.sigma_spin_select.value, self.sigma_kpoint_select.value,
                                             itemp=0, with_int_aw=True,
                                             ax_list=None, xlims=None, fontsize=8, **self.fig_kwargs)
        return self._mp(fig)

    def get_panel(self):
        """Return tabs with widgets to interact with the DDB file."""
        tabs = pn.Tabs(); app = tabs.append
        app(("Summary", pn.Row(bkw.PreText(text=self.sigeph.to_string(verbose=self.verbose),
                               sizing_mode="scale_both"))))
        #app(("e-Bands", pn.Row(self.get_plot_ebands_widgets(), self.on_plot_ebands_btn)))

        # Build different tabs depending on the calculation type.
        if self.sigeph.imag_only:
            app(("LWS", self.plot_lws))

        else:
            app(("QP-gaps", self.plot_qpgaps))
            app(("QP_vs_e0", self.plot_qps_vs_e0))
            app(("QPSolution", pn.Row(
                pn.Column("# Quasiparticle solution",
                          self.sigma_spin_select,
                          self.sigma_kpoint_select,
                          self.plot_plot_qpsolution_btn),
                self.plot_qpsolution_sk)
            ))

        return tabs



#!/usr/bin/env python

from abipy.abilab import RtaRobot

self = RtaRobot.from_dir(".")
print(self)

new_labels = ["k = q = " + str(abifile.ebands.kpoints.ksampling.mpdivs) for abifile in self.abifiles]
print(new_labels)

self.change_labels(new_labels)

import matplotlib.pyplot as plt
#self.plot_ibte_vs_rta_rho()
fig = self.plot_ibte_mrta_serta_conv(title="Al")
fig.suptitle("Al")
plt.show()



#!/usr/bin/env python
r"""
Phonon Band structures
======================

This example shows how to plot the phonon band structure of AlAs.
See tutorial/lesson_rf2.html
"""
from abipy.abilab import abiopen
import abipy.data as abidata

# Open the PHBST file produced by anaddb and get the phonon bands.
with abiopen(abidata.ref_file("trf2_5.out_PHBST.nc")) as ncfile:
    phbands = ncfile.phbands

# Read the Phonon DOS from the netcd file produced by anaddb (prtdos 2)
with abiopen(abidata.ref_file("trf2_5.out_PHDOS.nc")) as ncfile:
    phdos = ncfile.phdos

# plot phonon bands and DOS with LO-TO.
# `plot_phonons_lo_to.py` shows how to treat the LO-TO splitting.
phbands.plot(title="AlAs Phonon bands and DOS in eV")

# plot phonon bands with DOS.
phbands.plot_with_phdos(phdos, units="cm-1",
                        title="AlAs Phonon bands + DOS in cm-1")

# plot phonon DOS.
phdos.plot(units="cm-1", title="Phonon DOS and IDOS in cm-1")

# Plot the phonon band structure with different color for each line.
phbands.plot_colored_matched(units="cm-1",
                             title="AlAs with different color for each line.")



#!/usr/bin/env python
r"""
Band structure plot
===================

This example shows how to plot a band structure
using the eigenvalues stored in the GSR file
produced at the end of the GS run.
"""
from abipy.abilab import abiopen
import abipy.data as abidata

# Here we use one of the GSR files shipped with abipy.
# Replace filename with the path to your GSR file or your WFK file.
filename = abidata.ref_file("si_nscf_GSR.nc")

# Open the GSR file and extract the band structure.
# (alternatively one can use the shell and `abiopen.py OUT_GSR.nc -nb`
# to open the file in a jupyter notebook.
with abiopen(filename) as ncfile:
    ebands = ncfile.ebands

# Plot the band energies. Note that the labels for the k-points
# are found automatically in an internal database.
# Show fundamental and direct gaps.
#ebands.plot(with_gaps="fd", title="Silicon band structure")
ebands.plot(with_gaps=True, title="Silicon band structure")

# Plot the BZ and the k-point path.
ebands.kpoints.plot()



# coding: utf-8
"""PSPS file with tabulated data."""
import numpy as np

from collections import OrderedDict
from monty.bisect import find_gt
from monty.functools import lazy_property
from abipy.tools.plotting import add_fig_kwargs, get_ax_fig_plt, get_axarray_fig_plt
from abipy.iotools import ETSF_Reader
from abipy.core.mixins import AbinitNcFile

import logging
logger = logging.getLogger(__name__)


def mklabel(fsym, der, arg):
    """mklabel(f, 2, x) --> $f''(x)$"""
    if der == 0:
        return "$%s(%s)$" % (fsym, arg)
    else:
        fsym = fsym + "^{" + (der * r"\prime") + "}"
        return "$%s(%s)$" % (fsym, arg)


def rescale(arr, scale=1.0):
    if scale is None:
        return arr, 0.0

    amax = np.abs(arr).max()
    fact = scale / amax if amax != 0 else 1
    return fact * arr, fact


def dataframe_from_pseudos(pseudos, index=None):
    """
    Build pandas dataframe with the most important info associated to
    a list of pseudos or a list of objects that can be converted into pseudos.

    Args:
        pseudos: List of objects that can be converted to pseudos.
        index: Index of the dataframe.

    Return: pandas Dataframe.
    """
    from abipy.flowtk import PseudoTable
    pseudos = PseudoTable.as_table(pseudos)

    import pandas as pd
    attname = ["Z_val", "l_max", "l_local", "nlcc_radius", "xc", "supports_soc", "type"]
    rows = []
    for p in pseudos:
        row = OrderedDict([(k, getattr(p, k, None)) for k in attname])
        row["ecut_normal"], row["pawecutdg_normal"] = None, None
        if p.has_hints:
            hint = p.hint_for_accuracy(accuracy="normal")
            row["ecut_normal"] = hint.ecut
            if hint.pawecutdg: row["pawecutdg_normal"] = hint.pawecutdg
        rows.append(row)

    return pd.DataFrame(rows, index=index, columns=list(rows[0].keys()) if rows else None)


class PspsFile(AbinitNcFile):
    """
    Netcdf file with the tables used in Abinit to apply the
    pseudopotential part of the KS Hamiltonian.

    Usage example:

    .. code-block:: python

        with PspsFile("foo_PSPS.nc") as psps:
            psps.plot_tcore_rspace()
    """
    linestyles_der = ["-", "--", '-.', ':', ":", ":"]
    color_der = ["black", "red", "green", "orange", "cyan"]

    @classmethod
    def from_file(cls, filepath):
        """Initialize the object from a Netcdf file"""
        return cls(filepath)

    def __init__(self, filepath):
        super().__init__(filepath)
        self.reader = r = PspsReader(filepath)

    def close(self):
        """Close the file."""
        self.reader.close()

    @lazy_property
    def params(self):
        """:class:`OrderedDict` with parameters that might be subject to convergence studies."""
        return {}

    @add_fig_kwargs
    def plot(self, **kwargs):
        """
        Driver routine to plot several quantities on the same graph.

        Args:
            ecut_ffnl: Max cutoff energy for ffnl plot (optional)

        Return: |matplotlib-Figure|
        """
        methods = [
            "plot_tcore_rspace",
            "plot_tcore_qspace",
            "plot_ffspl",
            "plot_vlocq",
        ]

        ax_list, fig, plt = get_axarray_fig_plt(None, nrows=2, ncols=2,
                                                sharex=False, sharey=False, squeeze=True)

        ecut_ffnl = kwargs.pop("ecut_ffnl", None)
        for m, ax in zip(methods, ax_list.ravel()):
            getattr(self, m)(ax=ax, ecut_ffnl=ecut_ffnl, show=False)

        return fig

    @add_fig_kwargs
    def plot_tcore_rspace(self, ax=None, ders=(0, 1, 2, 3), rmax=3.0,  **kwargs):
        """
        Plot the model core and its derivatives in real space.

        Args:
            ax: |matplotlib-Axes| or None if a new figure should be created.
            ders: Tuple used to select the derivatives to be plotted.
            rmax: Max radius for plot in Bohr. None is full grid is wanted.

        Returns: |matplotlib-Figure|
        """
        ax, fig, plt = get_ax_fig_plt(ax=ax)

        linewidth = kwargs.pop("linewidth", 2.0)
        rmeshes, coresd = self.reader.read_coresd(rmax=rmax)

        scale = None
        scale = 1.0
        for rmesh, mcores in zip(rmeshes, coresd):
            for der, values in enumerate(mcores):
                if der not in ders: continue
                yvals, fact, = rescale(values, scale=scale)
                ax.plot(rmesh, yvals, color=self.color_der[der], linewidth=linewidth,
                        linestyle=self.linestyles_der[der],
                        label=mklabel("\\tilde{n}_c", der, "r") + " x %.4f" % fact)

        ax.grid(True)
        ax.set_xlabel("r [Bohr]")
        ax.set_title("Model core in r-space")
        if kwargs.get("with_legend", False): ax.legend(loc="best")

        return fig

    @add_fig_kwargs
    def plot_tcore_qspace(self, ax=None, ders=(0,), with_fact=True, with_qn=0, **kwargs):
        """
        Plot the model core in q space

        Args:
            ax: |matplotlib-Axes| or None if a new figure should be created.
            ders: Tuple used to select the derivatives to be plotted.
            with_qn:

        Returns: |matplotlib-Figure|
        """
        ax, fig, plt = get_ax_fig_plt(ax=ax)

        color = kwargs.pop("color", "black")
        linewidth = kwargs.pop("linewidth", 2.0)

        qmesh, tcore_spl = self.reader.read_tcorespl()
        #print(qmesh, tcore_spl)
        ecuts = 2 * (np.pi * qmesh)**2
        lines = []
        scale = 1.0
        scale = None
        for atype, tcore_atype in enumerate(tcore_spl):
            for der, values in enumerate(tcore_atype):
                if der == 1: der = 2
                if der not in ders: continue
                yvals, fact = rescale(values, scale=scale)

                label = mklabel("\\tilde{n}_{c}", der, "q")
                if with_fact: label += " x %.4f" % fact

                line, = ax.plot(ecuts, yvals, color=color, linewidth=linewidth,
                                linestyle=self.linestyles_der[der], label=label)
                lines.append(line)

                if with_qn and der == 0:
                    yvals, fact = rescale(qmesh * values, scale=scale)
                    line, ax.plot(ecuts, yvals, color=color, linewidth=linewidth,
                                  label=mklabel("q f", der, "q") + " x %.4f" % fact)

                    lines.append(line)

        ax.grid(True)
        ax.set_xlabel("Ecut [Hartree]")
        ax.set_title("Model core in q-space")
        if kwargs.get("with_legend", False): ax.legend(loc="best")

        return fig

    @add_fig_kwargs
    def plot_vlocq(self, ax=None, ders=(0,), with_qn=0, with_fact=True, **kwargs):
        """
        Plot the local part of the pseudopotential in q space.

        Args:
            ax: |matplotlib-Axes| or None if a new figure should be created.
            ders: Tuple used to select the derivatives to be plotted.
            with_qn:

        Returns: |matplotlib-Figure|
        """
        ax, fig, plt = get_ax_fig_plt(ax=ax)

        color = kwargs.pop("color", "black")
        linewidth = kwargs.pop("linewidth", 2.0)

        qmesh, vlspl = self.reader.read_vlspl()
        ecuts = 2 * (np.pi * qmesh)**2
        scale = 1.0
        scale = None
        for atype, vl_atype in enumerate(vlspl):
            for der, values in enumerate(vl_atype):
                if der == 1: der = 2
                if der not in ders: continue

                yvals, fact = rescale(values, scale=scale)
                label = mklabel("v_{loc}", der, "q")
                if with_fact: label += " x %.4f" % fact

                ax.plot(ecuts, yvals, color=color, linewidth=linewidth,
                        linestyle=self.linestyles_der[der], label=label)

                if with_qn and der == 0:
                    yvals, fact = rescale(qmesh * values, scale=scale)
                    ax.plot(ecuts, yvals, color=color, linewidth=linewidth,
                            label="q*f(q) x %2.f" % fact)

        ax.grid(True)
        ax.set_xlabel("Ecut [Hartree]")
        ax.set_title("Vloc(q)")
        if kwargs.get("with_legend", False): ax.legend(loc="best")

        return fig

    @add_fig_kwargs
    def plot_ffspl(self, ax=None, ecut_ffnl=None, ders=(0,), with_qn=0, with_fact=False, **kwargs):
        """
        Plot the nonlocal part of the pseudopotential in q-space.

        Args:
            ax: |matplotlib-Axes| or None if a new figure should be created.
            ecut_ffnl: Max cutoff energy for ffnl plot (optional)
            ders: Tuple used to select the derivatives to be plotted.
            with_qn:

        Returns: |matplotlib-Figure|
        """
        ax, fig, plt = get_ax_fig_plt(ax=ax)

        color = kwargs.pop("color", "black")
        linewidth = kwargs.pop("linewidth", 2.0)

        color_l = {-1: "black", 0: "red", 1: "blue", 2: "green", 3: "orange"}
        linestyles_n = ["solid", '-', '--', '-.', ":"]
        scale = None
        l_seen = set()

        qmesh, vlspl = self.reader.read_vlspl()

        all_projs = self.reader.read_projectors()
        for itypat, projs_type in enumerate(all_projs):
            # Loop over the projectors for this atom type.
            for p in projs_type:
                for der, values in enumerate(p.data):
                    if der == 1: der = 2
                    if der not in ders: continue
                    #yvals, fact = rescale(values, scale=scale)
                    label = None
                    if p.l not in l_seen:
                        l_seen.add(p.l)
                        label = mklabel("v_{nl}", der, "q") + ", l=%d" % p.l

                    stop = len(p.ecuts)
                    if ecut_ffnl is not None:
                        stop = find_gt(p.ecuts, ecut_ffnl)

                    #values = p.ekb * p.values - vlspl[itypat, 0, :]
                    values = vlspl[itypat, 0, :] + p.sign_sqrtekb * p.values

                    #print(values.min(), values.max())
                    ax.plot(p.ecuts[:stop], values[:stop], color=color_l[p.l], linewidth=linewidth,
                            linestyle=linestyles_n[p.n], label=label)

        ax.grid(True)
        ax.set_xlabel("Ecut [Hartree]")
        ax.set_title("ffnl(q)")
        if kwargs.get("with_legend", False): ax.legend(loc="best")

        ax.axhline(y=0, linewidth=linewidth, color='k', linestyle="solid")
        fig.tight_layout()

        return fig

    @add_fig_kwargs
    def compare(self, others, **kwargs):
        """Produce matplotlib plot comparing self with another list of pseudos ``others``."""
        if not isinstance(others, (list, tuple)):
            others = [others]

        ax_list, fig, plt = get_axarray_fig_plt(None, nrows=2, ncols=2,
                                                sharex=False, sharey=False, squeeze=True)
        ax_list = ax_list.ravel()
        #fig.suptitle("%s vs %s" % (self.basename, ", ".join(o.basename for o in others)))

        def mkcolor(count):
            npseudos = 1 + len(others)
            if npseudos <= 2:
                return {0: "red", 1: "blue"}[count]
            else:
                cmap = plt.get_cmap("jet")
                return cmap(float(count) / (1 + len(others)))

        ic = 0; ax = ax_list[ic]
        self.plot_tcore_rspace(ax=ax, color=mkcolor(0), show=False, with_legend=False)
        for count, other in enumerate(others):
            other.plot_tcore_rspace(ax=ax, color=mkcolor(count+1), show=False, with_legend=False)

        ic += 1; ax = ax_list[ic]
        self.plot_tcore_qspace(ax=ax, with_qn=0, color=mkcolor(0), show=False)
        for count, other in enumerate(others):
            other.plot_tcore_qspace(ax=ax, with_qn=0, color=mkcolor(count+1), show=False)

        ic += 1; ax = ax_list[ic]
        self.plot_vlocq(ax=ax, with_qn=0, color=mkcolor(0), show=False)
        for count, other in enumerate(others):
            other.plot_vlocq(ax=ax, with_qn=0, color=mkcolor(count+1), show=False)

        ic += 1; ax = ax_list[ic]
        self.plot_ffspl(ax=ax, with_qn=0, color=mkcolor(0), show=False)
        for count, other in enumerate(others):
            other.plot_ffspl(ax=ax, with_qn=0, color=mkcolor(count+1), show=False)

        return fig


class PspsReader(ETSF_Reader):
    """
    This object reads the results stored in the PSPS file produced by ABINIT.
    It provides helper function to access the most important quantities.
    """
    def __init__(self, filepath):
        super().__init__(filepath)

        # Get important quantities.
        self.usepaw, self.useylm = self.read_value("usepaw"), self.read_value("useylm")
        assert self.usepaw == 0 and self.useylm == 0
        self.ntypat = self.read_dimvalue("ntypat")
        self.lmnmax = self.read_dimvalue("lmnmax")
        self.indlmn = self.read_value("indlmn")

        self.znucl_typat = self.read_value("znucltypat")
        self.zion_typat = self.read_value("ziontypat")

        # TODO
        #self.psps_files = []
        #for strng in r.read_value("filpsp"):
        #    s = "".join(strng)
        #    print(s)
        #    self.psps_files.append(s)
        #print(self.psps_files)

    def read_coresd(self, rmax=None):
        """
        Read the core charges and derivatives for the different types of atoms.

        Args:
            rmax: Maximum radius in Bohr. If None, data on the full grid is returned.

        Returns:
            meshes: List of ntypat arrays. Each array contains the linear meshes in real space.
            coresd: List with nytpat arrays of shape [6, npts].

            (np.zeros. np.zeros) if core charge is not present

        xccc1d[ntypat6,n1xccc*(1-usepaw)]

        Norm-conserving psps only
        The component xccc1d(n1xccc,1,ntypat) is the pseudo-core charge
        for each type of atom, on the radial grid. The components
        xccc1d(n1xccc,ideriv,ntypat) give the ideriv-th derivative of the
        pseudo-core charge with respect to the radial distance.
        """

        xcccrc = self.read_value("xcccrc")
        try:
            all_coresd = self.read_value("xccc1d")
        except self.Error:
            # model core may not be present!
            return self.ntypat * [np.linspace(0, 6, num=100)], self.ntypat * [np.zeros((2, 100))]

        npts = all_coresd.shape[-1]
        rmeshes, coresd = [], []
        for itypat, rc in enumerate(xcccrc):
            rvals, step = np.linspace(0, rc, num=npts, retstep=True)
            ir_stop = -1
            if rmax is not None:
                # Truncate mesh
                ir_stop = min(int(rmax / step), npts) + 1
                #print(rmax, step, ir_stop, npts)

            rmeshes.append(rvals[:ir_stop])
            coresd.append(all_coresd[itypat, :, :ir_stop])

        return rmeshes, coresd

    def read_tcorespl(self):
        """
        Returns:
            qmesh: Linear q-mesh in G-space
            tcorespl:

        tcorespl[ntypat, 2, mqgrid_vl]
        Gives the pseudo core density in reciprocal space on a regular grid.
        Only if has_tcore
        """
        return self.read_value("qgrid_vl"), self.read_value("nc_tcorespl")

    def read_vlspl(self):
        """
        Returns:
            qmesh: Linear q-mesh in G-space
            vlspl:

        vlspl[2, ntypat, mqgrid_vl]
        Gives, on the radial grid, the local part of each type of psp.
        """
        return self.read_value("qgrid_vl"), self.read_value("vlspl")

    def read_projectors(self):
        """
        ffspl(ntypat, lnmax, 2, mqgrid_ff]
        Gives, on the radial grid, the different non-local projectors,
        in both the norm-conserving case, and the PAW case
        """
        # ekb(dimekb,ntypat*(1-usepaw))
        ekb = self.read_value("ekb")
        qgrid_ff = self.read_value("qgrid_ff")
        ffspl = self.read_value("ffspl")
        #print("qgrid", qgrid_ff.min(), qgrid_ff.max())

        projs = self.ntypat * [None]
        for itypat in range(self.ntypat):
            projs_type = []
            ln_list = self.get_lnlist_for_type(itypat)
            for i, ln in enumerate(ln_list):
                #print(ffspl[itypat, i, :, :])
                p = VnlProjector(itypat, ln, ekb[itypat, i], qgrid_ff, ffspl[itypat, i, :, :])
                projs_type.append(p)

            projs[itypat] = projs_type

        return projs

    def get_lnlist_for_type(self, itypat):
        """Return a list of (l, n) indices for this atom type."""
        # indlmn(6,lmn_size,ntypat)=array giving l,m,n,lm,ln,s for i=lmn
        indlmn_type = self.indlmn[itypat, :, :]

        iln0 = 0; ln_list = []
        for ilmn in range(self.lmnmax):
            iln = indlmn_type[ilmn, 4]
            if iln > iln0:
                iln0 = iln
                l = indlmn_type[ilmn, 0]  # l
                n = indlmn_type[ilmn, 2]  # n
                ln_list.append((l, n))

        return ln_list


class VnlProjector(object):
    """Data and parameters associated to a non-local projector."""
    def __init__(self, itypat, ln, ekb, qmesh, data):
        """
        Args:
            itypat:
            ln: Tuple with l and n.
            ekb: KB energy in Hartree.
            qmesh: Mesh of q-points.
            data: numpy array [2, nqpt]
        """
        self.ln = ln
        self.l, self.n, self.ekb = ln[0], ln[1], ekb
        self.qmesh, self.data = qmesh, data

        assert len(self.qmesh) == len(self.values)
        assert len(self.qmesh) == len(self.der2)

    @property
    def values(self):
        """Values of the projector in q-space."""
        return self.data[0, :]

    @property
    def der2(self):
        """Second order derivative."""
        return self.data[1, :]

    @property
    def ecuts(self):
        """List of cutoff energies corresponding to self.qmesh."""
        return 2 * (np.pi * self.qmesh)**2

    @property
    def sign_sqrtekb(self):
        return np.sign(self.ekb) * np.sqrt(np.abs(self.ekb))



# coding: utf-8
"""Decorators."""
import time

from functools import wraps


def return_straceback_ifexc(func):
    """
    Decorator for functions that are supposed to return a string for logging purposes (e.g. str)
    Instead of raising an exception, the decorated function returns a string with the
    traceback so that execution can continue.
    """
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception:
            import traceback
            return traceback.format_exc()
    return wrapper


def timeit(method):
    """
    timeit decorator adapted from:
    https://medium.com/pythonhive/python-decorator-to-measure-the-execution-time-of-methods-fa04cb6bb36d
    sets the timing of the routine as an attribute of the class
    """
    def timed(self, *args, **kw):
        ts = time.time()
        result = method(self, *args, **kw)
        te = time.time()

        setattr(self,"time_"+method.__name__, (te - ts) * 1000)
        return result
    return timed



"""Panels to interact with GSR files."""
import param
import panel as pn
import panel.widgets as pnw
import bokeh.models.widgets as bkw

from .core import PanelWithElectronBands, PanelWithEbandsRobot


class GsrFilePanel(PanelWithElectronBands):
    """
    Panel with widgets to interact with a |GsrFile|.
    """
    def __init__(self, gsr, **params):
        super().__init__(**params)
        self.gsr = gsr

    @property
    def ebands(self):
        """|ElectronBands|."""
        return self.gsr.ebands

    def get_panel(self):
        """Return tabs with widgets to interact with the DDB file."""
        tabs = pn.Tabs(); app = tabs.append
        app(("Summary", pn.Row(bkw.PreText(text=self.gsr.to_string(verbose=self.verbose),
                               sizing_mode="scale_both"))))
        app(("e-Bands", pn.Row(self.get_plot_ebands_widgets(), self.on_plot_ebands_btn)))

        # Add DOS tab only if k-sampling.
        kpoints = self.gsr.ebands.kpoints
        if kpoints.is_ibz:
            app(("e-DOS", pn.Row(self.get_plot_edos_widgets(), self.on_plot_edos_btn)))

            if self.gsr.ebands.supports_fermi_surface:
                # Fermi surface requires gamma-centered k-mesh
                app(("Fermi Surface", pn.Row(self.get_plot_fermi_surface_widgets(), self.on_plot_fermi_surface_btn)))

        return tabs


class GsrRobotPanel(PanelWithEbandsRobot):
    """
    A Panel to interoperate with multiple GSR files.
    """

    gsr_dataframe_btn = pnw.Button(name="Compute", button_type='primary')

    def __init__(self, robot, **params):
        super().__init__(**params)
        self.robot = robot

    @param.depends("gsr_dataframe_btn.clicks")
    def on_gsr_dataframe_btn(self):
        if self.gsr_dataframe_btn.clicks == 0: return
        df = self.robot.get_dataframe(with_geo=True)
        return pn.Column(self._df(df), sizing_mode='stretch_width')

    def get_panel(self):
        """Return tabs with widgets to interact with the |GsrRobot|."""
        tabs = pn.Tabs(); app = tabs.append
        app(("Summary", pn.Row(bkw.PreText(text=self.robot.to_string(verbose=self.verbose),
                               sizing_mode="scale_both"))))
        app(("e-Bands", pn.Row(self.get_ebands_plotter_widgets(), self.on_ebands_plotter_btn)))

        # Add e-DOS tab only if all ebands have k-sampling.
        if all(abifile.ebands.kpoints.is_ibz for abifile in self.robot.abifiles):
            app(("e-DOS", pn.Row(self.get_edos_plotter_widgets(), self.on_edos_plotter_btn)))

        app(("GSR-DataFrame", pn.Row(self.gsr_dataframe_btn, self.on_gsr_dataframe_btn)))

        return tabs



#!/usr/bin/env python
r"""
Projected phonon DOS
====================

This example shows how to plot the projected phonon DOS of AlAs.
See tutorial/lesson_rf2.html
"""
from abipy.abilab import abiopen
import abipy.data as abidata

# Read the Phonon DOS from the netcdf file produced by anaddb with prtdos 2
# (alternatively one can use the shell and `abiopen.py OUT_PHDOS.nc -nb`
# to open the file in a jupyter notebook.
with abiopen(abidata.ref_file("trf2_5.out_PHDOS.nc")) as phdos_file:

    # Plot PJDOS.
    phdos_file.plot_pjdos_type(units="cm-1", title="AlAs type-projected phonon DOS")

    # To have the projection along the cartesian directions (summed over atomic types)
    phdos_file.plot_pjdos_cartdirs_type(units="Thz", stacked=True,
            title="Type-projected ph-DOS decomposed along the three Cartesian directions.")

    # To plot the PJDOS for all the inequivalent sites.
    phdos_file.plot_pjdos_cartdirs_site(view="inequivalent", stacked=True)



"""Panels for interacting with output files in text format."""

#import param
import panel as pn
#import panel.widgets as pnw
import bokeh.models.widgets as bkw
from abipy.panels.core import AbipyParameterized


class AbinitOutputFilePanel(AbipyParameterized):
    """
    Panel with widgets to interact with an Abinit output file.
    """
    def __init__(self, outfile, **params):
        super().__init__(**params)
        self.outfile = outfile

    def _get_gridbox(self, what):
        """Return GridBox with matplotlib for GS/DFPT SCF cycles."""
        if what == "GS":
            cycles = self.outfile.get_all_gs_scf_cycles()
        elif what == "DFPT":
            cycles = self.outfile.get_all_d2de_scf_cycles()
        else:
            raise ValueError("Invalid value for what: %s" % what)

        if not cycles: return None

        num_plots, nrows, ncols = len(cycles), 1, 1
        if num_plots > 1:
            ncols = 2
            nrows = (num_plots // ncols) + (num_plots % ncols)

        box = pn.GridBox(nrows=nrows, ncols=ncols) #, sizing_mode='scale_both')
        for icycle, cycle in enumerate(cycles):
            box.append(self._mp(cycle.plot(title="%s cycle #%d" % (what, icycle), **self.fig_kwargs)))

        return box

    def get_panel(self):
        """Return tabs with widgets to interact with the Abinit output file."""
        tabs = pn.Tabs(); app = tabs.append
        app(("Summary", pn.Row(bkw.PreText(text=self.outfile.to_string(verbose=self.verbose),
                               sizing_mode="scale_both"))))
        df = self.outfile.get_dims_spginfo_dataframe().transpose()
        df.index.name = "Dataset"
        app(("Dims", self._df(df)))

        # Add tabs with plots for the GS/DFPT SCF cycles.
        for what in ("GS", "DFPT"):
            box = self._get_gridbox(what)
            if box is not None:
                app(("%s Cycles" % what, box))

        #timer = self.get_timer()
        #timer.plot_all(**self.fig_kwargs)

        return tabs



""""Panels for HIST files."""
import param
import panel as pn
import bokeh.models.widgets as bkw

from abipy.panels.core import sizing_mode_select, AbipyParameterized


_what_list = ["pressure", "forces", "energy", "abc", "angles", "volume"]


class HistFilePanel(AbipyParameterized):
    """
    Panel with widgets to interact with a |HistFile|.
    """
    what_list = pn.widgets.CheckBoxGroup(name="Select", value=_what_list, options=_what_list, inline=False)
    plot_relax_btn = pn.widgets.Button(name="Show relaxation", button_type="primary")

    sizing_mode = sizing_mode_select(value="stretch_width")

    appname = pn.widgets.Select(name="Viewer", value="ovito", options=["ovito", "mayavi", "vtk"])
    to_unit_cell = pn.widgets.Checkbox(name="To unit cell")
    view_relax_btn = pn.widgets.Button(name="View relaxation", button_type="primary")

    def __init__(self, hist, **params):
        super().__init__(**params)
        self.hist = hist

    def get_plot_relax_widgets(self):
        """Widgets to visualize the structure relaxation."""
        return pn.Column(self.what_list, self.sizing_mode, self.plot_relax_btn)

    @param.depends('plot_relax_btn.clicks')
    def on_plot_relax_btn(self):
        """
        Plot the evolution of structural parameters (lattice lengths, angles and volume)
        as well as pressure, info on forces and total energy.
        """
        if self.plot_relax_btn.clicks == 0: return

        num_plots, nrows, ncols = len(self.what_list.value), 1, 1
        if num_plots > 1:
            ncols = 2
            nrows = (num_plots // ncols) + (num_plots % ncols)

        box = pn.GridBox(nrows=nrows, ncols=ncols, sizing_mode=self.sizing_mode.value) #'scale_width')
        for i, what in enumerate(self.what_list.value):
            irow, icol = divmod(i, ncols)
            box.append(self._mp(self.hist.plot(what, title=what, **self.fig_kwargs)))

        return box
        #return pn.Column(box, box.controls(jslink=True))

    @param.depends('view_relax_btn.clicks')
    def on_view_relax_btn(self):
        if self.view_relax_btn.clicks == 0: return
        return self.hist.visualize(appname=self.appname.value, to_unit_cell=self.to_unit_cell.value)

    def get_panel(self):
        """Return tabs with widgets to interact with the DDB file."""
        tabs = pn.Tabs()
        tabs.append(("Summary", pn.Row(bkw.PreText(text=self.hist.to_string(verbose=self.verbose),
                     sizing_mode="scale_both"))))
        tabs.append(("Relaxation", pn.Row(self.get_plot_relax_widgets(), self.on_plot_relax_btn)))
        tabs.append(("Visualize", pn.Row(pn.Column(self.appname, self.to_unit_cell, self.view_relax_btn),
                                         self.on_view_relax_btn)))

        return tabs



#!/usr/bin/env python
"""
This script runs all the python scripts located in this directory
"""
import sys
import os
import argparse
import shutil
import tempfile

from subprocess import call
from abipy import __version__
from abipy import flowtk


def main():
    def str_examples():
        examples = """
          Usage example:\n\n
          runall.py => Run all scripts.
        """
        return examples

    def show_examples_and_exit(err_msg=None, error_code=1):
        """Display the usage of the script."""
        sys.stderr.write(str_examples())
        if err_msg:
            sys.stderr.write("Fatal Error\n" + err_msg + "\n")
        sys.exit(error_code)

    parser = argparse.ArgumentParser(epilog=str_examples(),formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('-V', '--version', action='version', version="%(prog)s version " + __version__)
    parser.add_argument('--loglevel', default="ERROR", type=str,
                        help="set the loglevel. Possible values: CRITICAL, ERROR (default), WARNING, INFO, DEBUG")

    parser.add_argument('-m', '--mode', type=str, default="sequential", help="execution mode. Default is sequential.")

    parser.add_argument('-e', '--exclude', type=str, default="", help="Exclude scripts. Comma-separated names")

    parser.add_argument('-x', '--execute', default=False, action="store_true", help="Execute flows.")

    parser.add_argument('--keep-dirs', action="store_true", default=False,
                        help="Do not remove flowdirectories.")

    parser.add_argument('-b', '--bail-on-failure', default=False, help="Exit at the first error.")

    #parser.add_argument("scripts", nargs="+",help="List of scripts to be executed")

    options = parser.parse_args()

    # loglevel is bound to the string value obtained from the command line argument.
    # Convert to upper case to allow the user to specify --loglevel=DEBUG or --loglevel=debug
    import logging
    numeric_level = getattr(logging, options.loglevel.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError('Invalid log level: %s' % options.loglevel)
    logging.basicConfig(level=numeric_level)

    # Find scripts.
    if options.exclude:
        options.exclude = options.exclude.split(",")
        print("Will exclude:\n", options.exclude)

    root = os.path.abspath(os.path.join(os.path.dirname(__file__), "flows"))
    scripts = []
    for fname in os.listdir(root):
        if fname in options.exclude: continue
        if fname.endswith(".py") and fname.startswith("run_"):
            path = os.path.join(root, fname)
            if path != __file__:
                scripts.append(path)
    print("Executing: %d scripts with mode: `%s` and execute: `%s`" % (
          len(scripts), options.mode, options.execute))

    # Run scripts according to mode.
    dirpaths, errors, retcode, cnt = [], [], 0, 0
    if options.mode in ["s", "sequential"]:
        for script in scripts:
            # flow will be produced in a temporary workdir.
            workdir = tempfile.mkdtemp(prefix='flow_' + os.path.basename(script))
            ret = call(["python", script, "--workdir", workdir])
            retcode += ret

            if ret != 0:
                e = "python %s returned retcode !=0" % script
                print(e)
                errors.append(e)
                if options.bail_on_failure:
                    print("Exiting now since bail_on_failure")
                    break

            dirpaths.append(workdir)

            # Here we execute the flow
            if options.execute:
                cnt += 1
                ret = 0
                try:
                    flow = flowtk.Flow.pickle_load(workdir)
                    flow.make_scheduler().start()
                    if not flow.all_ok: retcode += 1

                except Exception as exc:
                    ret += 1
                    s = "Exception raised during flow execution: %s\n:%s" % (flow, exc)
                    print(s)
                    errors.append(s)
                    if options.bail_on_failure:
                        print("Exiting now since bail_on_failure")
                        break
                    retcode += ret

        # Remove directories.
        if not options.keep_dirs:
            for dirpath in dirpaths:
                try:
                    shutil.rmtree(dirpath, ignore_errors=False)
                except OSError:
                    print("Exception while removing %s" % dirpath)

    else:
        show_examples_and_exit(err_msg="Wrong value for mode: %s" % options.mode)

    if errors:
        for i, err in enumerate(errors):
            print(92 * "=")
            print("[%d] %s" % (i, err))
            print(92 * "=")

    print("Number of scripts executed %d/%d" % (cnt, len(scripts)), ", final retcode %d" % retcode)
    return retcode


if __name__ == "__main__":
    sys.exit(main())



"""Panels for interacting with FATBANDS.nc files."""

#import param
import panel as pn
#import panel.widgets as pnw
import bokeh.models.widgets as bkw

from .core import PanelWithElectronBands #, PanelWithEbandsRobot


class FatBandsFilePanel(PanelWithElectronBands):
    """
    Panel with widgets to interact with a |FatBandsFile|.
    """
    def __init__(self, ncfile, **params):
        super().__init__(**params)
        self.ncfile = ncfile

    @property
    def ebands(self):
        """|ElectronBands|."""
        return self.ncfile.ebands

    def get_panel(self):
        """Return tabs with widgets to interact with the FATBANDS.nc file."""
        tabs = pn.Tabs(); app = tabs.append
        app(("Summary", pn.Row(bkw.PreText(text=self.ncfile.to_string(verbose=self.verbose),
                               sizing_mode="scale_both"))))
        app(("e-Bands", pn.Row(self.get_plot_ebands_widgets(), self.on_plot_ebands_btn)))

        if self.ncfile.ebands.kpoints.is_ibz:
            # Add DOS tab only if k-sampling.
            app(("e-DOS", pn.Row(self.get_plot_edos_widgets(), self.on_plot_edos_btn)))

            # Plot the L-PJDOS grouped by atomic type.
            #self.ncfile.plot_pjdos_typeview(lmax=lmax, **self.fig_kwargs)
            # Plot the L-PJDOS grouped by L.
            #self.ncfile.plot_pjdos_lview(lmax=lmax, **self.fig_kwargs)

            # Fermi surface requires a gamma-centered k-mesh
            if self.ncfile.ebands.supports_fermi_surface:
                app(("Fermi Surface", pn.Row(self.get_plot_fermi_surface_widgets(), self.on_plot_fermi_surface_btn)))

        elif self.ncfile.ebands.kpoints.is_path:
            # NC files have contributions up to L=4 (g channel)
            # but here we are intererested in s,p,d terms only so
            # we use the optional argument lmax
            lmax = 2

            # Plot the electronic fatbands grouped by atomic type.
            #self.ncfile.plot_fatbands_typeview(lmax=lmax, **self.fig_kwargs)
            # Plot the electronic fatbands grouped by L.
            #self.ncfile.plot_fatbands_lview(lmax=lmax, **self.fig_kwargs)

        else:
            raise ValueError("Neither a IBZ nor k-path!")

        return tabs


#class FatbandsRobotPanel(PanelWithEbandsRobot):
#    """
#    A Panel to interoperate with multiple GSR files.
#    """
#
#    gsr_dataframe_btn = pnw.Button(name="Compute", button_type='primary')
#
#    def __init__(self, robot, **params):
#        super().__init__(**params)
#        self.robot = robot
#
#    @param.depends("gsr_dataframe_btn.clicks")
#    def on_gsr_dataframe_btn(self):
#        if self.gsr_dataframe_btn.clicks == 0: return
#        df = self.robot.get_dataframe(with_geo=True)
#        return pn.Column(self._df(df), sizing_mode='stretch_width')
#
#    def get_panel(self):
#        """Return tabs with widgets to interact with the |GsrRobot|."""
#        tabs = pn.Tabs(); app = tabs.append
#        app(("Summary", pn.Row(bkw.PreText(text=self.robot.to_string(verbose=self.verbose),
#                               sizing_mode="scale_both"))))
#        app(("e-Bands", pn.Row(self.get_ebands_plotter_widgets(), self.on_ebands_plotter_btn)))
#
#        # Add e-DOS tab only if all ebands have k-sampling.
#        if all(abifile.ebands.kpoints.is_ibz for abifile in self.robot.abifiles):
#            app(("e-DOS", pn.Row(self.get_edos_plotter_widgets(), self.on_edos_plotter_btn)))
#
#        app(("GSR-DataFrame", pn.Row(self.gsr_dataframe_btn, self.on_gsr_dataframe_btn)))
#
#        return tabs



"""Inspired to http://wiki.wxpython.org/LongRunningTasks."""
from __future__ import print_function, division

import time
import wx

from threading import Thread

__all__ = [
"WorkerThread",
]

# Define notification event for thread completion
EVT_THREAD_RESULT_ID = wx.NewId()

def EVT_THREAD_RESULT(win, func):
    """Define Thread_Result Event."""
    win.Connect(-1, -1, EVT_THREAD_RESULT_ID, func)


class ThreadResultEvent(wx.PyEvent):
    """Simple event to carry arbitrary result data."""

    def __init__(self, **data):
        """Init Result Event."""
        wx.PyEvent.__init__(self)
        self.SetEventType(EVT_THREAD_RESULT_ID)
        self.data = data


class WorkerThread(Thread):
    """Worker Thread Class that executes processing."""

    def __init__(self, notify_window, group=None, target=None, name=None, *args, **kwargs):
        """Init Worker Thread Class."""
        super(WorkerThread,self).__init__(group=group, target=target, name=name, args=args, kwargs=kwargs)

        self._notify_window = notify_window
        self._want_abort = 0

    def start(self):
        """Start Worker Thread."""
        super(WorkerThread, self).start()

    def run(self):
        """Run Worker Thread."""
        # This is the code executing in the new thread. Simulation of
        # a long process (well, 10s here) as a simple loop - you will
        # need to structure your processing so that you periodically
        # peek at the abort variable
        retcode = super(WorkerThread, self).run()
        
        #while True:
        #    time.sleep(1)
        #    if self._want_abort:
        #        # Use a result of None to acknowledge the abort (of
        #        # course you can use whatever you'd like or even a separate event type)
        #        wx.PostEvent(self._notify_window, ThreadResultEvent(retcode="abort"))
        #        return

        # Here's where the result would be returned (this is an
        # example fixed result of the number 10, but it could be any Python object)
        wx.PostEvent(self._notify_window, ThreadResultEvent(retcode=retcode))
        return retcode 

    def abort(self):
        """Abort worker thread."""
        # Method for use by main thread to signal an abort
        self._want_abort = 1




#!/usr/bin/env python
r"""
MgB2 fatbands
=============

This example shows how to plot the L-projected fatbands of MgB2
using the FATBANDS.nc files produced by abinit with prtdos 3.
See also PhysRevLett.86.4656
"""
import abipy.abilab as abilab
import abipy.data as abidata

# Open the file (alternatively one can use the shell and `abiopen.py FILE -nb`
# to open the file in a jupyter notebook
# This file has been produced on a k-path so it's not suitable for DOS calculations.
fbnc_kpath = abilab.abiopen(abidata.ref_file("mgb2_kpath_FATBANDS.nc"))

# Print file info (dimensions, variables ...)
# Note that prtdos = 3, so LM decomposition is not available.
print(fbnc_kpath)


# Plot the k-points belonging to the path.
fbnc_kpath.ebands.kpoints.plot()

# NC files have contributions up to L=4 (g channel)
# but here we are intererested in s,p,d terms only so
# we use the optional argument lmax
lmax = 2

# Plot the electronic fatbands grouped by atomic type.
fbnc_kpath.plot_fatbands_typeview(lmax=lmax, tight_layout=True)

# Plot the electronic fatbands grouped by L.
fbnc_kpath.plot_fatbands_lview(lmax=lmax, tight_layout=True)

# Now we read another FATBANDS file produced on 18x18x18 k-mesh
fbnc_kmesh = abilab.abiopen(abidata.ref_file("mgb2_kmesh181818_FATBANDS.nc"))

print(fbnc_kmesh)
#fbnc_kmesh.ebands.kpoints.plot()

# Plot the L-PJDOS grouped by atomic type.
fbnc_kmesh.plot_pjdos_typeview(lmax=lmax, tight_layout=True)

# Plot the L-PJDOS grouped by L.
fbnc_kmesh.plot_pjdos_lview(lmax=lmax, tight_layout=True)

# Now we use the two netcdf files to produce plots with fatbands + PJDOSEs.
# The data for the DOS is taken from pjdosfile.
# sphinx_gallery_thumbnail_number = 6
fbnc_kpath.plot_fatbands_with_pjdos(pjdosfile=fbnc_kmesh, lmax=lmax,
                                    view="type", tight_layout=True)

# fatbands + PJDOS grouped by L
fbnc_kpath.plot_fatbands_with_pjdos(pjdosfile=fbnc_kmesh, lmax=lmax,
                                    view="lview", tight_layout=True)

fbnc_kpath.close()
fbnc_kmesh.close()



import abc
import os
import wx
import wx.lib.dialogs as wxdg
import abipy.gui.awx as awx
import abipy.gui.electronswx as ewx

from monty.os.path import which
from abipy.core.mixins import NcDumper
from abipy.iotools.visualizer import Visualizer
from abipy.electrons.ebands import ElectronBandsPlotter, ElectronDosPlotter
from abipy.gui.structure import StructureConverterFrame
from abipy.gui.converter import ConverterFrame
from abipy.gui.wxncview import NcViewerFrame
from abipy.dfpt.phonons import PhononBandsPlotter, PhononDosPlotter
from abipy.abilab import abiopen
from abipy.iotools import ETSF_Reader


class Has_Structure(metaclass=abc.ABCMeta):
    """
    Mixin class that provides a menu and callbacks
    for analyzing the crystalline structure.
    """
    @abc.abstractproperty
    def structure(self):
        """Structure object."""

    def CreateStructureMenu(self):
        """Creates the structure menu."""
        # Structure Menu ID's
        self.ID_STRUCT_CONVERT = wx.NewId()
        self.ID_STRUCT_VISUALIZE = wx.NewId()
        self.ID_STRUCT_SHOWBZ = wx.NewId()

        menu = wx.Menu()
        menu.Append(self.ID_STRUCT_CONVERT, "Convert", "Convert structure data to cif, POSCAR ...")
        self.Bind(wx.EVT_MENU, self.OnStructureConvert, id=self.ID_STRUCT_CONVERT)

        menu.Append(self.ID_STRUCT_SHOWBZ, "Show BZ", "Visualize the first Brillouin zone with matplotlib.")
        self.Bind(wx.EVT_MENU, self.OnStructureShowBz, id=self.ID_STRUCT_SHOWBZ)

        # Make sub-menu with the list of supported visualizers.
        visu_menu = wx.Menu()
        self._id2visuname = {}

        available_visus = [visu.name for visu in Visualizer.get_available()]

        for appname in available_visus:
            _id = wx.NewId()
            visu_menu.Append(_id, appname)
            self._id2visuname[_id] = appname
            self.Bind(wx.EVT_MENU, self.OnStructureVisualize, id=_id)

        menu.AppendMenu(-1, 'Visualize', visu_menu)
        return menu

    def OnStructureConvert(self, event):
        """Open new frame that allows the user to convert the structure."""
        StructureConverterFrame(self, self.structure).Show()

    def OnStructureVisualize(self, event):
        """"Call the visualizer to visualize the crystalline structure."""
        appname = self._id2visuname[event.GetId()]

        try:
            visu = self.structure.visualize(appname=appname)
            thread = awx.WorkerThread(self, target=visu)
            thread.start()

        except Exception:
            awx.showErrorMessage(self)

    def OnStructureShowBz(self, event):
        """"Visualize the Brillouin zone with matplotlib."""
        self.structure.show_bz()


class Has_Ebands(metaclass=abc.ABCMeta):
    """
    Mixin class that provides a menu and callbacks for analyzing electron bands.
    """
    @abc.abstractproperty
    def ebands(self):
        """`ElectronBands` object."""

    def CreateEbandsMenu(self):
        """Creates the ebands menu."""
        # Ebands Menu ID's
        self.ID_EBANDS_GETINFO = wx.NewId()
        self.ID_EBANDS_PLOT = wx.NewId()
        self.ID_EBANDS_DOS = wx.NewId()
        self.ID_EBANDS_JDOS = wx.NewId()
        #self.ID_EBANDS_FSURF = wx.NewId()
        #self.ID_EBANDS_SCISSORS = wx.NewId()

        menu = wx.Menu()

        menu.Append(self.ID_EBANDS_GETINFO, "Get Info", "Show info on the band structure")
        self.Bind(wx.EVT_MENU, self.OnEbandsGetInfo, id=self.ID_EBANDS_GETINFO)

        menu.Append(self.ID_EBANDS_PLOT, "Plot ebands", "Plot electron bands with matplotlib")
        self.Bind(wx.EVT_MENU, self.OnEbandsPlot, id=self.ID_EBANDS_PLOT)

        menu.Append(self.ID_EBANDS_DOS, "DOS", "Compute the electron DOS")
        self.Bind(wx.EVT_MENU, self.OnEbandsDos, id=self.ID_EBANDS_DOS)

        menu.Append(self.ID_EBANDS_JDOS, "JDOS", "Compute the electron Joint DOS")
        self.Bind(wx.EVT_MENU, self.OnEbandsJdos, id=self.ID_EBANDS_JDOS)

        # TODO
        #menu.Append(self.ID_EBANDS_FSURF, "Fermi surface", "Visualize the Fermi surface with Xcrysden")
        #self.Bind(wx.EVT_MENU, self.OnFermiSurface, id=self.ID_EBANDS_FSURF)

        # TODO
        #menu.Append(self.ID_EBANDS_SCISSORS, "Apply scissors", "Apply a scissors operator")
        #self.Bind(wx.EVT_MENU, self.OnApplyScissors, id=self.ID_EBANDS_SCISSORS)

        return menu

    def OnEbandsGetInfo(self, event):
        """Shows info on the bandstructure."""
        s = self.ebands.info
        caption = "Ebands info"
        wxdg.ScrolledMessageDialog(self, s, caption=caption, style=wx.MAXIMIZE_BOX).Show()

    def OnEbandsPlot(self, event):
        """Plot band energies with matplotlib."""
        self.ebands.plot()

    def OnEbandsDos(self, event):
        """Open Frame for the computation of the DOS."""
        ewx.ElectronDosFrame(self, bands=self.ebands).Show()

    def OnEbandsJdos(self, event):
        """Open Frame for the computation of the JDOS."""
        ewx.ElectronJdosFrame(self, bands=self.ebands).Show()

    def OnFermiSurface(self, event):
        """Visualize the Fermi surface with Xcrysden."""
        try:
            visu = self.ebands.export_bxsf(".bxsf")

            thread = awx.WorkerThread(self, target=visu)
            thread.start()

        except Exception:
            awx.showErrorMessage(self)

    #def OnApplyScissors(self, event):
    #    """
    #    Read the scissors operator from a pickle file, apply it to the electron bands and save the results.
    #    """
    #    Get the scissors operator from file
    #    Apply the scissors.
    #    new_ebands = self.ebands.apply_scissors(self, scissors)
    #    new_ebands.plot()
    #    new_ebands.pickle_dump()


class Has_MultipleEbands(Has_Ebands):
    """
    Mixin class that provides a menu and callbacks
    for analyzing and comparing multiple electron bands.
    """
    def CreateEbandsMenu(self):
        """Creates the ebands menu."""
        menu = super(Has_MultipleEbands, self).CreateEbandsMenu()
        menu.AppendSeparator()

        # Multiple Ebands Menu ID's
        self.ID_MULTI_EBANDS_PLOT = wx.NewId()
        self.ID_MULTI_EBANDS_DOS = wx.NewId()
        #self.ID_MULTI_EBANDS_JDOS = wx.NewId()
        self.ID_MULTI_EBANDS_BANDSWITHDOS = wx.NewId()

        menu.Append(self.ID_MULTI_EBANDS_PLOT, "Compare ebands", "Plot multiple electron bands")
        self.Bind(wx.EVT_MENU, self.OnCompareEbands, id=self.ID_MULTI_EBANDS_PLOT)
        menu.Append(self.ID_MULTI_EBANDS_DOS, "Compare DOSes", "Compare multiple electron DOSes")
        self.Bind(wx.EVT_MENU, self.OnCompareEdos, id=self.ID_MULTI_EBANDS_DOS)
        #menu.Append(self.ID_MULTI_EBANDS_JDOS, "Compare JDOSes", "Compare multiple electron JDOSes")
        #self.Bind(wx.EVT_MENU, self.OnCompareJdos, id=self.ID_MULTI_EBANDS_JDOS)

        menu.Append(self.ID_MULTI_EBANDS_BANDSWITHDOS, "Plot bands and DOS", "Plot electron bands and DOS on the same figure")
        self.Bind(wx.EVT_MENU, self.onPlotEbandsWithDos, id=self.ID_MULTI_EBANDS_BANDSWITHDOS)

        return menu

    @abc.abstractproperty
    def ebands_list(self):
        """Return a list of `ElectronBands`."""

    @abc.abstractproperty
    def ebands_filepaths(self):
        """
        Return a list with the absolute paths of the files
        from which the `ElectronBands` have been read.
        """

    def OnCompareEbands(self, event):
        """Plot multiple electron bands"""
        plotter = ElectronBandsPlotter()
        for path, ebands in zip(self.ebands_filepaths, self.ebands_list):
            label = os.path.relpath(path)
            plotter.add_ebands(label, ebands)

        try:
            print(plotter.bands_statdiff())
        except:
            pass
        plotter.plot()

    def OnCompareEdos(self, event):
        """Plot multiple electron DOSes"""
        # Open dialog to get DOS parameters.
        dialog = ewx.ElectronDosDialog(self)
        if dialog.ShowModal() == wx.ID_CANCEL: return
        dos_params = dialog.GetParams()

        plotter = ElectronDosPlotter()
        for path, ebands in zip(self.ebands_filepaths, self.ebands_list):
            try:
                edos = ebands.get_edos(**dos_params)
                label = os.path.relpath(path)
                plotter.add_edos(label, edos)
            except:
                awx.showErrorMessage(self)

        plotter.plot()

    #def OnCompareJdos(self, event):
        #"""Plot multiple electron JDOSes"""
        # Open dialog to get DOS parameters.
        #dialog = ElectronJdosDialog(self, nsppol, mband)
        #jdos_params = dialog.GetParams()

        #plotter = ElectronBandsPlotter()
        #for ebands in self.ebands_list:
        #    jos = ebands.get_jdos(**jdos_params)
        #    plotter.add_edos(label, edos)
        #
        #plotter.plot()

    def onPlotEbandsWithDos(self, event):
        """Plot electron bands with DOS. Requires the specification of two files."""
        # Open dialog to get files and DOS parameters.
        dialog = ewx.EbandsDosDialog(self, self.ebands_filepaths)
        if dialog.ShowModal() == wx.ID_CANCEL: return

        try:
            dos_params = dialog.getEdosParams()
            ipath, idos = dialog.getBandsDosIndex()

            ebands_path = self.ebands_list[ipath]
            ebands_mesh = self.ebands_list[idos]

            edos = ebands_mesh.get_edos(**dos_params)
            ebands_path.plot_with_edos(edos)
        except:
            awx.showErrorMessage(self)


class Has_Tools(object):
    """
    Mixin class that provides a menu with external tools.
    """
    def CreateToolsMenu(self):
        """Create the tools menu."""
        # Tools Menu ID's
        self.ID_TOOLS_UNIT_CONVERTER = wx.NewId()
        self.ID_TOOLS_PERIODIC_TABLE = wx.NewId()

        menu = wx.Menu()
        menu.Append(self.ID_TOOLS_PERIODIC_TABLE, "Periodic table", "Periodic Table")
        self.Bind(wx.EVT_MENU, self.OnTools_PeriodicTable, id=self.ID_TOOLS_PERIODIC_TABLE)

        menu.Append(self.ID_TOOLS_UNIT_CONVERTER, "Unit converter", "Unit Converter")
        self.Bind(wx.EVT_MENU, self.OnTools_UnitConverter, id=self.ID_TOOLS_UNIT_CONVERTER)

        return menu

    def OnTools_PeriodicTable(self, event):
        """Open new frame with the periodic table."""
        from awx.elements_gui import WxPeriodicTable
        WxPeriodicTable(self).Show()

    def OnTools_UnitConverter(self, event):
        """Open new frame with the unit converter."""
        ConverterFrame(self).Show()


class Has_NetcdfFiles(metaclass=abc.ABCMeta):
    """
    Mixin class that provides a menu and callbacks
    for analyzing and comparing netcdf files.
    """
    @abc.abstractproperty
    def nc_filepaths(self):
        """List of absolute paths of the netcdf file."""

    def CreateNetcdfMenu(self):
        """Creates the ebands menu."""
        # Netcdf Menu ID's
        self.ID_NETCDF_WXNCVIEW = wx.NewId()
        self.ID_NETCDF_NCDUMP = wx.NewId()
        self.ID_NETCDF_NCVIEW = wx.NewId()

        menu = wx.Menu()

        menu.Append(self.ID_NETCDF_WXNCVIEW, "wxncview", "Call wxncview")
        self.Bind(wx.EVT_MENU, self.OnNetcdf_WxNcView, id=self.ID_NETCDF_WXNCVIEW)

        menu.Append(self.ID_NETCDF_NCDUMP, "ncdump", "Show the output of ncdump")
        self.Bind(wx.EVT_MENU, self.OnNetcdf_NcDump, id=self.ID_NETCDF_NCDUMP)

        menu.Append(self.ID_NETCDF_NCVIEW, "ncview", "Call ncview")
        self.Bind(wx.EVT_MENU, self.OnNetcdf_NcView, id=self.ID_NETCDF_NCVIEW)

        return menu

    def OnNetcdf_NcDump(self, event):
        """Call ncdump and show results in a dialog."""
        for path in self.nc_filepaths:
            s = NcDumper().dump(path)
            caption = "ncdump output for %s" % path
            wxdg.ScrolledMessageDialog(self, s, caption=caption, style=wx.MAXIMIZE_BOX).Show()

    def OnNetcdf_NcView(self, event):
        """Call ncview in an subprocess."""
        if which("ncview") is None:
            return awx.showErrorMessage(self, "Cannot find ncview in $PATH")

        for path in self.nc_filepaths:
            def target():
                os.system("ncview %s" % path)

            thread = awx.WorkerThread(self, target=target)
            thread.start()

    def OnNetcdf_WxNcView(self, event):
        """Open wxncview frame."""
        NcViewerFrame(self, filepaths=self.nc_filepaths).Show()


class Has_Phbands(metaclass=abc.ABCMeta):
    """
    Mixin class that provides a menu and callbacks for analyzing phonon bands.
    """
    @abc.abstractproperty
    def phbands(self):
        """`PhononBands` object."""

    def CreatePhbandsMenu(self):
        """Creates the ebands menu."""
        # Ebands Menu ID's
        self.ID_PHBANDS_ADD_DOS = wx.NewId()
        self.ID_PHBANDS_ADD_LO_TO = wx.NewId()
        self.ID_PHBANDS_PLOT = wx.NewId()
        self.ID_PHBANDS_DOS = wx.NewId()
        self.ID_MULTI_PHBANDS_BANDSWITHDOS = wx.NewId()

        menu = wx.Menu()

        menu.Append(self.ID_PHBANDS_ADD_DOS, "Add phdos data", "Add the phonon dos data from a PHDOS.nc file")
        self.Bind(wx.EVT_MENU, self.OnAddDos, id=self.ID_PHBANDS_ADD_DOS)

        menu.Append(self.ID_PHBANDS_ADD_LO_TO, "Add LO-TO data", "Add the LO-TO splitting data from a PHDOS.nc file")
        self.Bind(wx.EVT_MENU, self.OnAddLoTo, id=self.ID_PHBANDS_ADD_LO_TO)

        menu.Append(self.ID_PHBANDS_PLOT, "Plot phbands", "Plot phonon bands with matplotlib")
        self.Bind(wx.EVT_MENU, self.OnPhbandsPlot, id=self.ID_PHBANDS_PLOT)

        menu.Append(self.ID_PHBANDS_DOS, "DOS", "Plot the phonon DOS")
        self.Bind(wx.EVT_MENU, self.OnPhbandsDos, id=self.ID_PHBANDS_DOS)

        menu.Append(self.ID_MULTI_PHBANDS_BANDSWITHDOS, "Plot bands and DOS", "Plot phonon bands and DOS on the same figure")
        self.Bind(wx.EVT_MENU, self.onPlotPhbandsWithDos, id=self.ID_MULTI_PHBANDS_BANDSWITHDOS)

        return menu

    def OnAddDos(self, event):
        """Add PHDOS data to the active tab"""
        dialog = wx.FileDialog(self, message="Choose a PHDOS.nc file", defaultDir=os.getcwd(),
                       wildcard="Netcdf files (*.nc)|*.nc",
                       style=wx.OPEN | wx.CHANGE_DIR)

        if dialog.ShowModal() == wx.ID_CANCEL: return
        phdos = abiopen(dialog.GetPath())

        self.active_tab.phdos_file = phdos

    def OnAddLoTo(self, event):
        """Add LO-TO splitting data to the phbands in the active tab"""
        dialog = wx.FileDialog(self, message="Choose an anaddb.nc file", defaultDir=os.getcwd(),
                       wildcard="Netcdf files (*.nc)|*.nc",
                       style=wx.OPEN | wx.CHANGE_DIR)

        if dialog.ShowModal() == wx.ID_CANCEL: return

        self.phbands.read_non_anal_from_file(dialog.GetPath())

    def OnPhbandsPlot(self, event):
        """Plot phonon frequencies with matplotlib."""
        self.phbands.plot()

    def OnPhbandsDos(self, event):
        """Open Frame for the computation of the DOS."""
        if not self.phdos:
            awx.showErrorMessage(self, message="PHDOS data should be loaded using the menu Phband->Add phdos data")
        else:
            plotter = PhononDosPlotter()
            try:
                label = os.path.relpath(self.active_phdos_file.filepath)
                plotter.add_phdos(label, self.phdos)
            except:
                awx.showErrorMessage(self)
            plotter.plot()

    def onPlotPhbandsWithDos(self, event):
        """Plot phonon bands with DOS"""

        try:
            if not self.phdos:
                awx.showErrorMessage(self, message="PHDOS data should be loaded using the menu Phband->Add phdos data")
            else:
                self.phbands.plot_with_phdos(self.phdos)
        except:
            awx.showErrorMessage(self)

    @abc.abstractproperty
    def phdos(self):
        """PHDOS data for the active tab if it has been added. None otherwise"""


class Has_MultiplePhbands(Has_Phbands):
    """
    Mixin class that provides a menu and callbacks
    for analyzing and comparing multiple phonon bands.
    """
    def CreatePhbandsMenu(self):
        """Creates the ebands menu."""
        menu = super(Has_MultiplePhbands, self).CreatePhbandsMenu()
        menu.AppendSeparator()

        # Multiple Ebands Menu ID's
        self.ID_MULTI_PHBANDS_PLOT = wx.NewId()
        self.ID_MULTI_PHBANDS_DOS = wx.NewId()

        menu.Append(self.ID_MULTI_PHBANDS_PLOT, "Compare phbands", "Plot multiple phonon bands")
        self.Bind(wx.EVT_MENU, self.OnComparePhbands, id=self.ID_MULTI_PHBANDS_PLOT)
        menu.Append(self.ID_MULTI_PHBANDS_DOS, "Compare DOSes", "Compare multiple phonon DOSes")
        self.Bind(wx.EVT_MENU, self.OnComparePhdos, id=self.ID_MULTI_PHBANDS_DOS)

        return menu

    @abc.abstractproperty
    def phbands_list(self):
        """Return a list of `PhononBands`."""

    @abc.abstractproperty
    def phbands_filepaths(self):
        """
        Return a list with the absolute paths of the files
        from which the `PhononBands` have been read.
        """

    @abc.abstractproperty
    def phdos_list(self):
        """Return a list of `PhononDos`."""

    @abc.abstractproperty
    def phdos_filepaths(self):
        """
        Return a list with the absolute paths of the files
        from which the `PhononDos` have been read.
        """

    def OnComparePhbands(self, event):
        """Plot multiple phonon bands"""

        dialog = ewx.BandsCompareDialog(self, self.phbands_filepaths)
        if dialog.ShowModal() == wx.ID_CANCEL: return

        try:
            selected = dialog.GetSelectedIndices()

        except:
            awx.showErrorMessage(self)

        plotter = PhononBandsPlotter()

        for i in selected:
            label = os.path.relpath(self.phbands_filepaths[i])
            plotter.add_phbands(label, self.phbands_list[i])

        try:
            print(plotter.bands_statdiff())
        except:
            pass
        plotter.plot()

    def OnComparePhdos(self, event):
        """Plot multiple phonon DOSes"""

        plotter = PhononDosPlotter()
        for path, phdos in zip(self.phdos_filepaths, self.phdos_list):
            try:
                label = os.path.relpath(path)
                plotter.add_phdos(label, phdos)
            except:
                awx.showErrorMessage(self)

        plotter.plot()



"""Collections of simple frames used to solve typical (simple) problems."""
from __future__ import print_function, division

import wx

from abipy.gui.awx.core import Frame as awxFrame

__all__ = [
    "FrameWithChoice",
]


class FrameWithChoice(wx.Frame):
    """
    Simple frame with a Choice control and two buttons: OK, Cancel
    Subclasses will usually redefine onOkButton.
    """
    def __init__(self, parent, choices, **kwargs):
        super(FrameWithChoice, self).__init__(parent, id=-1, **kwargs)

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        panel = wx.Panel(self, -1)

        self.wxchoices = wx.Choice(panel, -1, wx.DefaultPosition, wx.DefaultSize, list(choices), 0)
        self.wxchoices.SetSelection(0)

        main_sizer.Add(self.wxchoices, flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL | wx.EXPAND, border=5)

        ok_button = wx.Button(panel, wx.ID_OK, label='Ok')
        ok_button.Bind(wx.EVT_BUTTON, self.onOkButton)

        close_button = wx.Button(panel, wx.ID_CANCEL, label='Cancel')
        close_button.Bind(wx.EVT_BUTTON, self.onCloseButton)

        hbox = wx.BoxSizer(wx.HORIZONTAL)

        hbox.Add(ok_button,  0, flag=wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, border=5)
        hbox.Add(close_button,  0, flag=wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, border=5)
        main_sizer.Add(hbox, 0, flag=wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, border=5)

        panel.SetSizerAndFit(main_sizer)
        #self.SetSizerAndFit(main_sizer)

    def getChoice(self):
        """Returns the string selected by the user."""
        return self.wxchoices.GetStringSelection()

    def onOkButton(self, event):
        print("In onOkButton with choice selected %s" % self.wxchoices.GetStringSelection())

    def onCloseButton(self, event):
        self.Destroy()


if __name__ == "__main__":
    app = wx.App()
    FrameWithChoice(None, ["hello", "ciao"]).Show()
    app.MainLoop()



import os
import wx
import wx.lib.agw.flatnotebook as fnb
import abipy.gui.awx as awx

from wx.py.shell import Shell
from monty.string import marquee
from abipy.abilab import abiopen
from abipy.electrons.bse import MdfPlotter
from abipy.iotools.visualizer import Visualizer
from abipy.gui.kpoints import KpointsPanel
from abipy.gui import mixins as mix
from abipy.gui.baseviewer import MultiViewerFrame


# TODO Add ebands to MDF.nc
class MdfViewerFrame(MultiViewerFrame, mix.Has_Structure, mix.Has_MultipleEbands, mix.Has_Tools, mix.Has_NetcdfFiles):
#class MdfViewerFrame(MultiViewerFrame, mix.Has_Structure, mix.Has_Tools, mix.Has_NetcdfFiles):
    VERSION = "0.1"

    HELP_MSG = """Quick help:

 Toolbar:
    Click the button to plot the spectrum (averaged over the q-points)
    Use the combo boxes to select the type of spectrum and the quantity to plot.

 Qpoint list:

     Right-Click:  display popup menu with choices.
     Select:  plot the spectra for this qpoint.

Also, these key bindings can be used
(For Mac OSX, replace 'Ctrl' with 'Apple'):

  Ctrl-Q:     quit
"""
    @property
    def codename(self):
        return "MdfViewer"

    @property
    def active_mdf_file(self):
        """The active MDF file i.e. the MDF associated to the active tab."""
        return self.active_tab.mdf_file

    @property
    def structure(self):
        """`Structure` associated to the active tab."""
        return self.active_mdf_file.structure

    @property
    def ebands(self):
        """`ElectronBands` associated to the active tab."""
        return self.active_mdf_file.ebands

    @property
    def ebands_list(self):
        """List of `ElectronBands`."""
        ebands_list = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            ebands_list.append(tab.mdf_file.ebands)
        return ebands_list

    @property
    def ebands_filepaths(self):
        """
        Return a list with the absolute paths of the files
        from which the `ElectronBands` have been read.
        """
        paths = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            paths.append(tab.mdf_file.filepath)
        return paths

    @property
    def nc_filepaths(self):
        """String with the absolute paths of the netcdf files."""
        paths = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            paths.append(tab.mdf_file.filepath)
        return paths

    @property
    def mdf_filepaths(self):
        """
        Return a list with the absolute paths of the files
        from which the `MDF_Files` have been read.
        """
        paths = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            paths.append(tab.mdf_file.filepath)
        return paths

    @property
    def mdf_files_list(self):
        """List of `MDF_Files`."""
        mdf_lists = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            mdf_lists.append(tab.mdf_file)
        return mdf_lists

    def makeMenu(self):
        """Creates the main menu."""
        # Base menu.
        menu_bar = super(MdfViewerFrame, self).makeMenu()

        # Add Mixin menus.
        menu_bar.Append(self.CreateStructureMenu(), "Structure")
        #menu_bar.Append(self.CreateEbandsMenu(), "Ebands")
        menu_bar.Append(self.CreateMdfMenu(), "Mdf")
        menu_bar.Append(self.CreateToolsMenu(), "Tools")
        menu_bar.Append(self.CreateNetcdfMenu(), "Netcdf")

        # Help menu
        help_menu = self.makeHelpMenu()
        menu_bar.Append(help_menu, "Help")

        self.SetMenuBar(menu_bar)

    def CreateMdfMenu(self):
        # MDF Menu ID's
        self.ID_MDF_PLOT_AVERAGE = wx.NewId()
        self.ID_MDF_COMPARE = wx.NewId()

        menu = wx.Menu()
        menu.Append(self.ID_MDF_PLOT_AVERAGE, "Plot averaged MDF", "Plot the average of the macroscopic dielectric function")
        self.Bind(wx.EVT_MENU, self.onPlotAveragedMdf, id=self.ID_MDF_PLOT_AVERAGE)

        menu.AppendSeparator()

        menu.Append(self.ID_MDF_COMPARE, "Compare MDF", "Compare multiple macroscopic dielectric functions")
        self.Bind(wx.EVT_MENU, self.OnMdfCompare, id=self.ID_MDF_COMPARE)

        return menu

    def onPlotAveragedMdf(self, event):
        """Plot the average of the macroscopic dielectric function"""
        mdf_type = self.getMdfType()
        cplx_mode = self.getCplxMode()
        #print(mdf_type, cplx_mode)

        mdf_file = self.active_mdf_file
        mdf_file.plot_mdfs(cplx_mode=cplx_mode, mdf_type=mdf_type)

    def OnMdfCompare(self, event):
        """Compare multiple averaged macroscopic dielectric functions"""
        mdf_type = self.getMdfType()
        cplx_mode = self.getCplxMode()

        if mdf_type == "ALL":
            return awx.showErrorMessage(self, "ALL is not supported by Compare. Please use EXC, RPA, GWRPA")

        plotter = MdfPlotter()
        for path, mdf_file in zip(self.mdf_filepaths, self.mdf_files_list):
            label = os.path.relpath(path)
            mdf = mdf_file.get_mdf(mdf_type)
            plotter.add_mdf(label, mdf)

        plotter.plot(cplx_mode, qpoint=None)

    def makeToolBar(self):
        """Creates the toolbar."""
        self.toolbar = toolbar = self.CreateToolBar()
        toolbar.SetToolBitmapSize(wx.Size(48, 48))

        def bitmap(path):
            return wx.Bitmap(awx.path_img(path))

        artBmp = wx.ArtProvider.GetBitmap
        toolbar.AddSimpleTool(wx.ID_OPEN, artBmp(wx.ART_FILE_OPEN, wx.ART_TOOLBAR), "Open")
        toolbar.AddSeparator()

        # Button to plot the averaged MDD
        # TODO: Change icon.
        toolbar.AddSimpleTool(self.ID_MDF_PLOT_AVERAGE, bitmap("wave.png"), "Plot averaged MDF")

        # Combo box with the list of MDF types
        mdf_types = ["ALL", "EXC", "RPA", "GWRPA"]
        self.mdftype_cbox = wx.ComboBox(toolbar, id=-1, name='MDF type', choices=mdf_types, value="ALL", style=wx.CB_READONLY)
        self.mdftype_cbox.SetToolTipString("Select the type of MDF spectra to plot.")
        toolbar.AddControl(control=self.mdftype_cbox, label="MDF Type:")

        # Combo box with the list of complex modes
        cplx_modes = ["Re-Im", "Re", "Im"]
        self.cplx_cbox = wx.ComboBox(toolbar, id=-1, name='COMPLEX mode', choices=cplx_modes, value="Re-Im", style=wx.CB_READONLY)
        self.cplx_cbox.SetToolTipString("Select the component of the MDF spectra to plot (real or imaginary part).")
        toolbar.AddControl(control=self.cplx_cbox, label="Complex Mode:")

        toolbar.Realize()

    def addFileTab(self, parent, filepath):
        """Open a file and add a new tab to the notebook."""
        mdf_file = abiopen(filepath)
        tab = MdfFileTab(self.notebook, mdf_file)
        self.notebook.AddPage(tab, os.path.basename(filepath))

        # List to events triggered by the popup menu in the qpoints_table.
        qpanel = tab.qpoints_panel
        self.Bind(qpanel.MYEVT_COMPARE_SPECTRAQ, self.onCompareSpectraQ)

    def getMdfType(self):
        """Return the sting with the MDF type selected by the user."""
        mdf_type = self.mdftype_cbox.GetStringSelection()
        if not mdf_type: mdf_type = "ALL"
        return mdf_type

    def getCplxMode(self):
        """Return the sting with the complex mode used for plotting the spectra."""
        cplx_mode = self.cplx_cbox.GetStringSelection()
        if not cplx_mode: cplx_mode = "Re-Im"
        return cplx_mode

    def onCompareSpectraQ(self, event):
        """
        Compare different MDF(q) spectra (provided that we have multiple tabs in the notebook).
        This callback is executed when MdfQpointsPanel fires CompareSpectraQEvent.
        """
        qpoint = event.qpoint
        mdf_type = self.getMdfType()
        cplx_mode = self.getCplxMode()

        if mdf_type == "ALL":
            return awx.showErrorMessage(self, "ALL is not supported by Compare. Please use EXC, RPA, GWRPA")

        plotter = MdfPlotter()

        # Extract the type of MDF we are interested in
        for path, mdf_file in zip(self.mdf_filepaths, self.mdf_files_list):
            label = os.path.relpath(path)
            mdf = mdf_file.get_mdf(mdf_type)
            plotter.add_mdf(label, mdf)

        plotter.plot(cplx_mode, qpoint=qpoint)


class MdfQpointsPanel(KpointsPanel):
    """Extend KpointsPanel adding popupmenus"""

    # Command event used to signal that the we should compare multiple spectra (fixed q, move along files).
    CompareSpectraQEvent, MYEVT_COMPARE_SPECTRAQ = wx.lib.newevent.NewCommandEvent()

    def __init__(self, parent, mdf_file, **kwargs):
        KpointsPanel.__init__(self, parent, mdf_file.structure, mdf_file.qpoints)
        self.parent = parent
        self.mdf_file = mdf_file

        # Connect the events fired by klist_ctrl.
        self.klist_ctrl.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.onQpointActivated)

    def makePopupMenu(self):
        """Build the popup menu."""
        # Menu of the base class
        menu = super(MdfQpointsPanel, self).makePopupMenu()

        # Add other options
        self.ID_POPUP_MDF_QCOMPARE = wx.NewId()
        menu.Append(self.ID_POPUP_MDF_QCOMPARE, "Compare spectra(q)")

        # Associate menu/toolbar items with their handlers.
        menu_handlers = [
            (self.ID_POPUP_MDF_QCOMPARE, self.onCompareSpectraQ),
        ]

        for combo in menu_handlers:
            mid, handler = combo[:2]
            self.Bind(wx.EVT_MENU, handler, id=mid)

        return menu

    @property
    def viewer_frame(self):
        """The parent frame `MdfViewerFrame`."""
        try:
            return self._viewer_frame

        except AttributeError:
            self._viewer_frame = self.getParentWithType(MdfViewerFrame)
            return self._viewer_frame

    def onQpointActivated(self, event):
        """Plot MDF(q) for the selected q-point."""
        qpoint = self.getSelectedKpoint()
        if qpoint is None: return

        # Get the options from the ViewerFrame.
        mdf_type = self.viewer_frame.getMdfType()
        cplx_mode = self.viewer_frame.getCplxMode()

        #print(qpoint)
        self.mdf_file.plot_mdfs(cplx_mode=cplx_mode, mdf_type=mdf_type, qpoint=qpoint)

    def onCompareSpectraQ(self, event):
        """
        Get the selected q-point and fire CompareSpectraQEvent.
        The parent frame will detect the event and will compare the
        different MDF spectra (provided that we have multiple tabs in the notebook.
        """
        qpoint = self.getSelectedKpoint()
        if qpoint is None: return
        # Post the event.
        event = self.CompareSpectraQEvent(id=-1, qpoint=qpoint)
        wx.PostEvent(self.parent, event)


class MdfFileTab(awx.Panel):
    """Tab showing information on a single MDF file."""
    def __init__(self, parent, mdf_file, **kwargs):
        """
        Args:
            parent:
                parent window.
            mdf_file:
        """
        super(MdfFileTab, self).__init__(parent, -1, **kwargs)
        self.mdf_file = mdf_file

        splitter = wx.SplitterWindow(self, id=-1, style=wx.SP_3D)
        splitter.SetSashGravity(0.95)

        self.qpoints_panel = MdfQpointsPanel(splitter, mdf_file)

        # Add Python shell
        msg = "MDF_object is accessible via the mdf_file variable. Use mdf_file.<TAB> to access the list of methods."
        msg = marquee(msg, width=len(msg) + 8, mark="#")
        msg = "#"*len(msg) + "\n" + msg + "\n" + "#"*len(msg) + "\n"

        # FIXME <Error>: CGContextRestoreGState: invalid context 0x0
        pyshell = Shell(splitter, introText=msg, locals={"mdf_file": mdf_file})
        splitter.SplitHorizontally(self.qpoints_panel, pyshell)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(splitter, 1, wx.EXPAND, 5)
        self.SetSizerAndFit(sizer)

    @property
    def statusbar(self):
        return self.viewer_frame.statusbar

    @property
    def viewer_frame(self):
        """The parent frame `MdfViewerFrame`."""
        try:
            return self._viewer_frame

        except AttributeError:
            self._viewer_frame = self.getParentWithType(MdfViewerFrame)
            return self._viewer_frame


class MdfViewerApp(awx.App):
    pass


def wxapp_mdfviewer(mdf_filepaths):
    """Standalone application."""
    app = MdfViewerApp()
    frame = MdfViewerFrame(None, filepaths=mdf_filepaths)
    app.SetTopWindow(frame)
    frame.Show()
    return app




# coding: utf-8
"""Work subclasses related to GS calculations."""
#import json

from .works import Work
#from abipy.core.structure import Structure


class CompareGsTasksWork(Work):
    """
    A work made of different GsTasks. Each task perform a GS calculation
    for the same system but different options for the GS solver.
    """

    def on_all_ok(self):
        """
        This method is called when all tasks reach S_OK. It reads the energies
        and the volumes from the GSR file, computes the EOS and produce a
        JSON file `eos_data.json` in outdata.
        """
        self.getnwrite_eosdata()
        return dict(returncode=0, message="EOS computed and file written")



# coding: utf-8
"""Work subclasses related to GS calculations."""
import json

from .works import Work
from abipy.core.structure import Structure


class EosWork(Work):
    """
    Work for the computation of the Equation of State
    The EOS is obtained by computing E(V) for several volumes around the input V0,
    The initial volumes are obtained by rescaling the input lattice vectors so that
    length proportions and angles are preserved.
    This guess is exact for cubic materials while other Bravais lattices require
    a constant-volume optimization of the cell geometry.

    If lattice_type=="cubic" and atomic positions are fixed by symmetry.
    use can use move_atoms=False to perform standard GS-SCF calculations.
    In all the other cases, E(V) is obtained by relaxing the atomic positions at fixed volume.

    The E(V) points are fitted at the end of the work and the results are saved in the
    `eos_data.json` file produced in the `outdata` directory.
    The file contains the energies, the volumes and the values of V0, B0, B1 obtained
    with different EOS models.
    """

    @classmethod
    def from_scf_input(cls, scf_input, npoints=4, deltap_vol=0.25, ecutsm=0.5, move_atoms=True,
                       manager=None):
        """
        Build a EosWork from an AbinitInput representing a SCF-GS calculation.

        Args:
            scf_input: AbinitInput for SCF-GS used as template to generate the other inputs.
            npoints: Number of volumes generated on the right (left) of the equilibrium volume
                The total number of points is therefore 2 * n + 1.
            deltap_vol: Step of the linear mesh given in relative percentage of the equilibrium volume
                The step is thus: v0 * deltap_vol / 100.
            ecutsm: Value of ecutsm input variable. If `scf_input` does not provide ecutsm, this
                value will be used else the vale in `scf_input`.
            move_atoms: If True, a structural relaxation of ions is performed for each volume
                This is needed if the atomic positions are non fixed by symmetry.
            manager: TaskManager instance. Use default if None.

        Return: EosWork instance.
        """
        new_work = cls(manager=manager)

        structure = scf_input.structure
        lattice_type = structure.spget_lattice_type()
        assert lattice_type is not None

        dvol = structure.volume * deltap_vol / 100
        v0 = structure.volume - dvol * npoints
        new_work.input_volumes = [v0 + ipt * dvol for ipt in range(2 * npoints + 1)]

        if "ecutsm" not in scf_input:
            print("Input does not define ecutsm input variable.\n",
                  "A default value of %s will be added to all the EOS inputs" % ecutsm)

        for vol in new_work.input_volumes:
            # Build structure with new volume and generate new input.
            new_lattice = structure.lattice.scale(vol)
            new_structure = Structure(new_lattice, structure.species, structure.frac_coords)
            new_input = scf_input.new_with_structure(new_structure)

            # Add ecutsm if not already present.
            new_input.set_vars_ifnotin(ecutsm=ecutsm)

            if lattice_type == "cubic" and not move_atoms:
                # Perform GS calculations without moving atoms, cells do not need to be relaxed.
                new_input.pop_vars(["ionmov", "optcell", "ntime"])
                new_work.register_scf_task(new_input)
            else:
                # Constant-volume optimization of cell geometry + atoms.
                # (modify acell and rprim under constraint - normalize the vectors of rprim to generate the acell)
                # In principle one could take into account the symmetry of the lattice...
                new_input.set_vars_ifnotin(ionmov=2, ntime=50, optcell=3, dilatmx=1.05)
                new_work.register_relax_task(new_input)

        return new_work

    def getnwrite_eosdata(self, write_json=True):
        """
        This method is called when all tasks reach S_OK. It reads the energies
        and the volumes from the GSR file, computes the EOS and produce a
        JSON file `eos_data.json` in outdata.
        """
        energies_ev, volumes = [], []
        for task in self:
            with task.open_gsr() as gsr:
                volumes.append(float(gsr.structure.volume))
                energies_ev.append(float(gsr.energy))

        from pymatgen.analysis.eos import EOS
        eos_data = {"input_volumes_ang3": self.input_volumes,
                    "volumes_ang3": volumes,
                    "energies_ev": energies_ev}

        for model in EOS.MODELS:
            if model in ("deltafactor", "numerical_eos"): continue
            try:
                fit = EOS(model).fit(volumes, energies_ev)
                eos_data[model] = {k: float(v) for k, v in fit.results.items()}
            except Exception as exc:
                eos_data[model] = {"exception": str(exc)}

        if write_json:
            with open(self.outdir.path_in("eos_data.json"), "wt") as fh:
                json.dump(eos_data, fh, indent=4, sort_keys=True)

        return eos_data

    def on_all_ok(self):
        """
        This method is called when all tasks reach S_OK. It reads the energies
        and the volumes from the GSR file, computes the EOS and produce a
        JSON file `eos_data.json` in outdata.
        """
        self.getnwrite_eosdata()
        return dict(returncode=0, message="EOS computed and file written")


#class FiniteDiffStressWork(Work):
#    """
#    Work for the computation of the stress tensor
#    """
#
#    @classmethod
#    def from_scf_input(cls, scf_input, delta=1e-4, ecutsm=0.5, manager=None):
#        """
#        Build a EosWork from an AbinitInput representing a SCF-GS calculation.
#
#        Args:
#            scf_input: AbinitInput for SCF-GS used as template to generate the other inputs.
#	    delta:
#            ecutsm: Value of ecutsm input variable. If `scf_input` does not provide ecutsm, this
#                value will be used else the vale in `scf_input`.
#            manager: TaskManager instance. Use default if None.
#
#        Return: EosWork instance.
#        """
#	scf_input = scf_input.deepcopy()
#
#        if "ecutsm" not in scf_input:
#            scf_input.set_vars(ecutsm=ecutsm)
#            print("Input does not define ecutsm input variable.\n",
#                  "A default value of %s will be added to all the EOS inputs" % ecutsm)
#
#        new_work = cls(manager=manager)
#        self.unstrained_task = new_work.register_scf_task(scf_input)
#
#	voigt_inds = [(0, 0)]
#
#	self.delta = float(delta)
#	self.tasks_voigt = {}
#	for voigt in voigt_inds:
#	    self.tasks_voigt[voigt] = defaultdict(list)
#	    for isign in (-1, +1):
#		# Apply strain to the lattice.
#                strain = np.zeros((3, 3))
#                strain[voigt] = float(isign) * delta
#                new_structure = scf_input.structure.deepcopy()
#                new_structure.apply_strain(strain)
#                new_input = scf_input.new_with_structure(new_structure)
#
#                # Perform GS calculations with strained cell.
#                task = new_work.register_scf_task(new_input)
#                self.tasks_voigt[voigt].append(task)
#
#        return new_work
#
#    def compute_and_write_stress(self):
#        """
#        This method is called when all tasks reach S_OK.
#	It reads the energies and the volumes from the GSR file, computes the EOS and produce a
#        JSON file `eos_data.json` in outdata.
#        """
#	with self.unstrained_task.open_gsr() as gsr0:
#	    e0, v0 = gsr0.energy, gsr0.structure.volume
#	    cart_stress_tensor = gsr0.cart_stress_tensor
#
#	fd_tensor = np.empty((3, 3))
#	for voigt in voigt_inds:
#	    tasks = self.tasks_voigt[voigt]
#	    energies_ev, volumes = np.empty(len(tasks), np.empty(len(tasks))
#	    for i, task in enumerate(task):
#                with task.open_gsr() as gsr:
#                   energies_ev[i] = float(gsr.energy))
#
#	    d = (e_plus - e_minus) / (2 * self.delta * v0)
#            fd_tensor[voigt] = d
#            fd_tensor[voigt[1], voigt[0]] = d
#
#	data = {
#             "dfpt_cart_stress_tensor": cart_stress_tensor,
#             "finite_diff_cart_stress_tensor": fd_cart_stress_tensor,
#        }
#
#        with open(self.outdir.path_in("stress.json"), "wt") as fh:
#            json.dump(data, fh, indent=4, sort_keys=True)
#
#        return data
#
#    def on_all_ok(self):
#        """
#        This method is called when all tasks reach S_OK. It reads the energies
#        and the volumes from the GSR file, computes the EOS and produce a
#        JSON file `eos_data.json` in outdata.
#        """
#        self.compute_and_write_stress()
#        return dict(returncode=0, message="EOS computed and file written")



import os
import wx
import abc
import wx.lib.agw.flatnotebook as fnb
import abipy.gui.awx as awx

from monty.string import list_strings


class MultiViewerFrame(awx.Frame, metaclass=abc.ABCMeta):
    """
    Base class for Viewers that can handle the multiple netcdf files
    of the same type. A `MultiViewerFrame` has a notebook where
    each page provides tools to interact with the data stored in the netcd file
    Concrete classes should provide the method that open a new file and creates
    a new page (specific to the file type) that will be added to the notebook.

    Concrete classes must also define the following (class) attributes.

    .. attributes:

        VERSION:
            String with the version of the visualizer e.g  "0.1"
        HELP_MSG:
            Multi line string with a Quick help. For example a list of shortcuts
            or a brief description of the usage of the GUI.
    """
    def __init__(self, parent, filepaths=(), **kwargs):
        """
        Args:
            parent:
                parent window.
            filepaths:
                String or list of strings with the path of the netcdf files to open
                Empty tuple if no file should be opened during the initialization of the frame.
        """
        super(MultiViewerFrame, self).__init__(parent, -1, title=self.codename, **kwargs)

        # This combination of options for config seems to work on my Mac.
        self.config = wx.FileConfig(appName=self.codename, localFilename=self.codename + ".ini", 
                                    style=wx.CONFIG_USE_LOCAL_FILE)

        # Build menu, toolbar and status bar.
        self.makeMenu()
        self.makeToolBar()
        self.statusbar = self.CreateStatusBar()

        # Open netcdf files.
        filepaths, exceptions = list_strings(filepaths), []
        filepaths = map(os.path.abspath, filepaths)

        # Create the notebook (each file will have its own tab).
        panel = wx.Panel(self, -1)

        try:
            style = fnb.FNB_NO_X_BUTTON  | fnb.FNB_NAV_BUTTONS_WHEN_NEEDED
        except AttributeError:
            style = fnb.FNB_NO_X_BUTTON 

        self.notebook = fnb.FlatNotebook(panel, -1, style=style)
                                                                                           
        for path in filepaths:
            self.read_file(path)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.notebook, 1, wx.EXPAND, 5)
        panel.SetSizerAndFit(sizer)

    @abc.abstractproperty
    def codename(self):
        """The name of the viewer."""

    @property
    def active_tab(self):
        """Returns the active tab. None if notebook is empty."""
        return self.notebook.GetCurrentPage()

    def makeMenu(self):
        """
        Method of the base class that provides a base menu
        that can be extended by the subclass.
        """
        menu_bar = wx.MenuBar()

        file_menu = wx.Menu()
        file_menu.Append(wx.ID_OPEN, "&Open", help="Open an existing file in a new tab")
        file_menu.Append(wx.ID_CLOSE, "&Close", help="Close the file associated to the active tab")
        file_menu.Append(wx.ID_EXIT, "&Quit", help="Exit the application")

        file_history = self.file_history = wx.FileHistory(8)
        file_history.Load(self.config)
        recent = wx.Menu()
        file_history.UseMenu(recent)
        file_history.AddFilesToMenu()
        file_menu.AppendMenu(-1, "&Recent Files", recent)
        self.Bind(wx.EVT_MENU_RANGE, self.OnFileHistory, id=wx.ID_FILE1, id2=wx.ID_FILE9)
        menu_bar.Append(file_menu, "File")

        # Associate menu/toolbar items with their handlers.
        menu_handlers = [
            (wx.ID_OPEN, self.OnOpen),
            (wx.ID_CLOSE, self.OnClose),
            (wx.ID_EXIT, self.OnExit),
        ]
                                                            
        for combo in menu_handlers:
            mid, handler = combo[:2]
            self.Bind(wx.EVT_MENU, handler, id=mid)

        return menu_bar

    def makeHelpMenu(self):
        """
        Method of the base class that provides a base help menu
        that can be extended by the subclass.
        """
        help_menu = wx.Menu()
                                                                                                                 
        self.ID_HELP_QUICKREF = wx.NewId()
        help_menu.Append(self.ID_HELP_QUICKREF, "Quick Reference ", help="Quick reference for " + self.codename)
        help_menu.Append(wx.ID_ABOUT, "About " + self.codename, help="Info on the application")

        # Associate menu/toolbar items with their handlers.
        menu_handlers = [
            (self.ID_HELP_QUICKREF, self.onQuickRef),
            (wx.ID_ABOUT, self.OnAboutBox),
        ]
                                                            
        for combo in menu_handlers:
            mid, handler = combo[:2]
            self.Bind(wx.EVT_MENU, handler, id=mid)
                                                     
        return help_menu

    @abc.abstractmethod
    def makeToolBar(self):
        """To be provided by the concrete class."""

    @abc.abstractmethod
    def addFileTab(self, parent, filepath):
        """
        This method must be provided by the subclass. 
        It receives a string with the file path, opens the file
        and adds a new tab to the notebook.

        Example::

            wfk = abiopen(filepath)
            tab = WfkFileTab(self.notebook, wfk)
            self.notebook.AddPage(tab, os.path.basename(filepath))
        """

    def read_file(self, filepath):
        """Open netcdf file, create new tab and save the file in the history."""
        self.statusbar.PushStatusText("Reading %s" % filepath)
        try:
            self.addFileTab(self, filepath)
            # don't know why but this does not work!
            self.notebook.Refresh()
            self.notebook.SetSelection(self.notebook.GetPageCount())
            self.AddFileToHistory(filepath)
        except:
            awx.showErrorMessage(self)

    def OnOpen(self, event):
        """Open FileDialog to allow the user to select a file."""
        # intercept possible problems when determining the current directory (e.g. it has been deleted)
        try:
            def_dir = os.getcwd()
        except OSError:
            def_dir = os.path.expanduser('~')
        # Show the dialog and retrieve the user response.
        # If it is the OK response, process the data.
        dialog = wx.FileDialog(self, message="Choose a netcdf file", defaultDir=def_dir,
                               wildcard="Netcdf files (*.nc)|*.nc",
                               style=wx.OPEN | wx.CHANGE_DIR)

        if dialog.ShowModal() == wx.ID_CANCEL: return 
        self.read_file(dialog.GetPath())

    def OnClose(self, event):
        """
        Remove the active tab from the notebook and 
        close the corresponding netcdf file, 
        """
        notebook = self.notebook
        if notebook.GetPageCount() == 0: return
        idx = notebook.GetSelection()
        if idx == -1: return None
                                                                                          
        # Close the file
        tab = notebook.GetPage(idx)

        # Remove tab.
        notebook.DeletePage(idx)
        notebook.Refresh()
        #notebook.SendSizeEvent()
                                                                                          
    def OnExit(self, event):
        """Exits the application."""
        # Close open netcdf files.
        #try:
        #    for index in range(self.notebook.GetPageCount()):
        #        tab = self.notebook.GetPage(index)
        #        try:
        #            tab.wfk.close()
        #        except:
        #            pass
        #finally:
        self.Destroy()
                                                                                          
    def OnAboutBox(self, event):
        """"Info on the application."""
        awx.makeAboutBox(codename=self.codename, version=self.VERSION,
                         description="", developers="M. Giantomassi")

    def onQuickRef(self, event=None):
        """Show a dialog with a brief description of the commands."""
        dialog = wx.MessageDialog(self, self.HELP_MSG, self.codename + " Quick Reference",
                               wx.OK | wx.ICON_INFORMATION)
        dialog.ShowModal()
        dialog.Destroy()

    def AddFileToHistory(self, filepath):
        """Add the absolute filepath to the file history."""
        self.file_history.AddFileToHistory(filepath)
        self.file_history.Save(self.config)
        self.config.Flush()

    def OnFileHistory(self, event):
        fileNum = event.GetId() - wx.ID_FILE1
        filepath = self.file_history.GetHistoryFile(fileNum)
        self.read_file(filepath)



#!/usr/bin/env python
r"""
Speed of Sound
==============

This example shows how to compute the speed of sound by fitting phonon frequencies
along selected directions by linear least-squares fit.
For a command line interface use:

    abiview.py ddb_vs input_DDB
"""
from __future__ import print_function

import os
import abipy.data as abidata

from abipy import abilab
from abipy.dfpt.vsound import SoundVelocity

# Initialize object from DDB file.
ddb_path = os.path.join(abidata.dirpath, "refs", "si_sound_vel", "Si_DDB")
sv = SoundVelocity.from_ddb(ddb_path)

# Get pandas dataframe with results.
df = sv.get_dataframe()
abilab.print_dataframe(df)

# Plot fit
sv.plot()



from __future__ import print_function, division

import wx

__all__ = [
    "ConsoleEvent",
    "ConsoleWindow",
]

#######################################################################
# Events for the console widget
#######################################################################

EVT_CONSOLE_TYPE = wx.NewEventType()
EVT_CONSOLE = wx.PyEventBinder(EVT_CONSOLE_TYPE, 1)


class ConsoleEvent(wx.PyEvent):
    """
    This event is triggering for messages that
    should be displayed within the console.
    """
    def __init__(self, kind, msg):
        wx.PyEvent.__init__(self)
        self.SetEventType(EVT_CONSOLE_TYPE)
        self.kind, self.msg = kind, msg


class ConsoleWindow(wx.TextCtrl):
    """
    This serves as the primary console for displaying commands and initialization information.
    The console window is color coded. To write to the console window, widgets should post console events.
    """
    # Message kinds
    INFO = "info"
    DEBUG = "debug"
    WARNING = "warning"
    CRITICAL = "critical"
    ERROR = 'error'

    ALL_KINDS =[INFO, DEBUG, WARNING, CRITICAL, ERROR]

    def __init__(self, parent):
        """Creates the console window."""
        wx.TextCtrl.__init__(self, parent, -1, style=wx.TE_MULTILINE | wx.TE_READONLY | wx.NO_BORDER | wx.TE_RICH2)

        self.idPOPUP_CLEAR = wx.NewId()

        # "kind > %s\n",
        self.format = {}
        for kind in self.ALL_KINDS:
            self.format[kind] = kind + "> %s\n"

        self.colors = {k: None for k in self.ALL_KINDS}
        self.wxColors = {k: None for k in self.ALL_KINDS}

        # Set ourselves to receive all console events
        self.Bind(EVT_CONSOLE, self.onConsoleEvent)

        # Set up the console menu handler
        self.Bind(wx.EVT_RIGHT_DOWN, self.onRightClick, self)

    def displayBanner(self, **kwargs):
        """Displays the application startup banner."""
        self.insertText('===============================================')
        self.insertText("Welcome to program %s" % version)
        self.insertText('===============================================')
        self.insertText('')

    def insertText(self, text, kind=None):
        """
        Writes text to the console window in the appropriate color for the kind of message.
        """
        self.SetDefaultStyle(wx.TextAttr(self.getColor(kind)))
        self.AppendText(self.format[kind] % text)

    def getColor(self, kind):
        """Gets the wx color object for the specified text kind."""
        #config = utils.getApplicationConfiguration()
        #current_color = config[kind + '_color']
        if kind is None:
            return None

        wx_color = None

        #current_color = "error"
        #if self.colors[kind] == current_color:
        #    wx_color = self.wxColors[kind]
        #else:
        #    wx_color = apply(wx.Colour, current_color)
        #    self.colors[kind] = current_color
        #    self.wxColors[kind] = wx_color

        return wx_color

    def onConsoleEvent(self, event):
        """
        Processes a console event by displaying the contents of the event to the console window.
        """
        self.insertText(event.msg, event.kind)

    def onRightClick(self, event):
        """Handles a right click by displaying the popupmenu at the click point."""
        menu = self.makePopupMenu()
        self.PopupMenu(menu, event.GetPosition())

    def makePopupMenu(self):
        """Creates the popup menu for operations on the console window."""
        menu = wx.Menu()
        menu.Append(self.idPOPUP_CLEAR, "Clear Console")

        self.Bind(wx.EVT_MENU, self.onClear, id=self.idPOPUP_CLEAR)
        return menu

    def onClear(self, event):
        """Processes a clearing of the console window event."""
        self.Clear()


class FrameWithConsole(wx.Frame):
    """
    Simple frame with a Choice control and two buttons: OK, Cancel
    Subclasses will usually redefine onOkButton.
    """
    def __init__(self, parent, choices, **kwargs):
        super(FrameWithConsole, self).__init__(parent, id=-1, **kwargs)
        self.parent = parent

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        panel = wx.Panel(self, -1)
        main_sizer.Add(panel, flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL | wx.EXPAND, border=5)

        ok_button = wx.Button(self, wx.ID_OK, label='Ok')
        ok_button.Bind(wx.EVT_BUTTON, self.onOkButton)

        close_button = wx.Button(self, wx.ID_CANCEL, label='Cancel')
        close_button.Bind(wx.EVT_BUTTON, self.onCloseButton)

        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(ok_button)
        hbox.Add(close_button, flag=wx.LEFT, border=5)

        main_sizer.Add(hbox, flag=wx.ALIGN_CENTER | wx.ALL | wx.EXPAND)

        splitter = wx.SplitterWindow(self, -1, style=wx.SP_3DSASH)
        self.console = ConsoleWindow(splitter)
        csplitter = wx.SplitterWindow(splitter, -1, style=wx.SP_3DSASH)
        # Set the default control window size
        splitter.SplitHorizontally(self.console, csplitter, 150)

        main_sizer.Add(splitter, 1, wx.EXPAND)
        #main_sizer.Add(self.console, flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL | wx.EXPAND, border=5)

        self.SetAutoLayout(True)
        self.SetSizerAndFit(main_sizer)

    def onOkButton(self, event):
        event = ConsoleEvent(kind="error", msg="Hello")
        #print("Firing %s" % event)
        wx.PostEvent(self.console, event)

    def onCloseButton(self, event):
        self.Destroy()


if __name__ == "__main__":
    app = wx.App()
    frame = FrameWithConsole(None, -1)
    frame.Show(True)
    app.SetTopWindow(frame)
    app.MainLoop()



#!/usr/bin/env python
r"""
Brillouin zone
==============

This example shows how to display the Brillouin zone
with pymatgen and matplotlib.
"""
from abipy.abilab import abiopen
import abipy.data as abidata

# Open the WKF file.
wfk_file = abiopen(abidata.ref_file("si_scf_WFK.nc"))

# Extract the crystalline structure.
structure = wfk_file.structure

# Visualize the BZ.
structure.plot_bz()

# Close the wfk file
wfk_file.close()



""""Panels for DDB files."""

import param
import panel as pn
import panel.widgets as pnw
import bokeh.models.widgets as bkw

from abipy.panels.core import AbipyParameterized


class DdbFilePanel(AbipyParameterized):
    """
    A panel to analyze a |DdbFile|.
    Provides widgets to invoke anaddb and visualize the results.
    """
    verbose = param.Integer(0, bounds=(0, None), doc="Verbosity Level")
    mpi_procs = param.Integer(1, bounds=(1, None), doc="Number of MPI processes used in anaddb")

    nqsmall = param.Integer(10, bounds=(1, None), doc="Number of divisions for smallest vector to generate Q-mesh")
    ndivsm = param.Integer(5, bounds=(1, None), doc="Number of divisions for smallest segment in q-path")
    lo_to_splitting = param.ObjectSelector(default="automatic", objects=["automatic", True, False])
    chneut = param.ObjectSelector(default=1, objects=[0, 1, 2], doc="Abinit variable")
    dipdip = param.ObjectSelector(default=1, objects=[0, 1], doc="Abinit variable")
    asr = param.ObjectSelector(default=2, objects=[0, 1, 2], doc="Abinit variable")
    units = param.ObjectSelector(default="eV", objects=["eV", "meV", "Ha", "cm-1", "Thz"], doc="Energy units")

    dos_method = param.ObjectSelector(default="tetra", objects=["tetra", "gaussian"], doc="Integration method for DOS")
    temp_range = pnw.RangeSlider(name="T-range", start=0.0, end=1000, value=(0.0, 300.0), step=20)

    gamma_ev = param.Number(1e-4, bounds=(1e-20, None), doc="Phonon linewidth in eV")
    w_range = pnw.RangeSlider(name="Frequency range (eV)", start=0.0, end=1.0,
                              value=(0.0, 0.1), step=0.001)

    get_epsinf_btn = pnw.Button(name="Compute", button_type='primary')
    plot_phbands_btn = pnw.Button(name="Plot Bands and DOS", button_type='primary')
    plot_eps0w_btn = pnw.Button(name="Plot eps0(omega)", button_type='primary')

    plot_vsound_btn = pnw.Button(name="Calculate speed of sound", button_type='primary')
    plot_check_asr_dipdip_btn = pnw.Button(name="Compute phonons with/wo ASR and DIPDIP", button_type='primary')
    plot_ifc_btn = pnw.Button(name="Compute IFC(R)", button_type='primary')

    def __init__(self, ddb, **params):
        super().__init__(**params)
        self.ddb = ddb

    @param.depends('get_epsinf_btn.clicks')
    def get_epsinf(self):
        """Compute eps_infinity and Born effective charges from DDB."""
        if self.get_epsinf_btn.clicks == 0: return

        epsinf, becs = self.ddb.anaget_epsinf_and_becs(chneut=self.chneut,
                                                       mpi_procs=self.mpi_procs, verbose=self.verbose)

        gen, inp = self.ddb.anaget_dielectric_tensor_generator(asr=self.asr, chneut=self.chneut, dipdip=self.dipdip,
                                                               mpi_procs=self.mpi_procs, verbose=self.verbose,
                                                               return_input=True)

        eps0 = gen.tensor_at_frequency(w=0, gamma_ev=self.gamma_ev)
        #eps0 = pnw.DataFrame(eps0.get_dataframe())
        return pn.Column(epsinf, eps0, becs, pn.pane.HTML(inp._repr_html_()))

    @param.depends('plot_eps0w_btn.clicks')
    def plot_eps0w(self):
        """Compute eps0(omega) from DDB and plot the results."""
        if self.plot_eps0w_btn.clicks == 0: return
        gen, inp = self.ddb.anaget_dielectric_tensor_generator(asr=self.asr, chneut=self.chneut, dipdip=self.dipdip,
                                                               mpi_procs=self.mpi_procs, verbose=self.verbose,
                                                               return_input=True)
        ws = self.w_range.value
        w_max = ws[1]
        if w_max == 1.0: w_max = None # Will compute w_max in plot routine from ph freqs.

        def p(component, reim):
            return gen.plot(w_min=ws[0], w_max=w_max, gamma_ev=self.gamma_ev, num=500, component=component,
                            reim=reim, units=self.units, **self.fig_kwargs)

        # Build grid
        gspec = pn.GridSpec(sizing_mode='scale_width')
        gspec[0, 0] = p("diag", "re")
        gspec[0, 1] = p("diag", "im")
        gspec[1, 0] = p("offdiag", "re")
        gspec[1, 1] = p("offdiag", "im")
        gspec[2, :] = gen.get_oscillator_dataframe(reim="all", tol=1e-6)
        # Add HTML pane with input.
        gspec[3, 0] = pn.pane.HTML(inp._repr_html_())

        return gspec

    @param.depends('plot_phbands_btn.clicks')
    def plot_phbands_and_phdos(self, event=None):
        """Compute phonon bands and ph-DOSes from DDB and plot the results."""
        if self.plot_phbands_btn.clicks == 0: return
        #self.plot_phbands_btn.button_type = "warning"

        #print("Computing phbands")
        with self.ddb.anaget_phbst_and_phdos_files(
                nqsmall=self.nqsmall, qppa=None, ndivsm=self.ndivsm,
                line_density=None, asr=self.asr, chneut=self.chneut, dipdip=self.dipdip,
                dos_method=self.dos_method, lo_to_splitting=self.lo_to_splitting,
                verbose=self.verbose, mpi_procs=self.mpi_procs, return_input=True) as g:

            phbst_file, phdos_file = g
            phbands, phdos = phbst_file.phbands, phdos_file.phdos
            #print("Computing phbands completed")

            # Build grid
            gspec = pn.GridSpec(sizing_mode='scale_width')
            gspec[0, 0] = phbands.plot_with_phdos(phdos, units=self.units, **self.fig_kwargs)
            gspec[0, 1] = phdos_file.plot_pjdos_type(units=self.units, exchange_xy=True, **self.fig_kwargs)
            gspec[1, 0] = phdos_file.msqd_dos.plot(units=self.units, **self.fig_kwargs)
            temps = self.temp_range.value
            gspec[1, 1] = phdos.plot_harmonic_thermo(tstart=temps[0], tstop=temps[1], num=50, **self.fig_kwargs)
            #msqd_dos.plot_tensor(**self.fig_kwargs)
            #self.plot_phbands_btn.button_type = "primary"

            # Add HTML pane with input
            gspec[2,:] = pn.pane.HTML(g.input._repr_html_())

        return gspec

    @param.depends('plot_vsound_btn.clicks')
    def plot_vsound(self):
        """
        Compute the speed of sound by fitting phonon frequencies
        along selected directions by linear least-squares fit.
        """
        if self.plot_vsound_btn.clicks == 0: return
        from abipy.dfpt.vsound import SoundVelocity
        sv = SoundVelocity.from_ddb(self.ddb.filepath, num_points=20, qpt_norm=0.1,
                                    ignore_neg_freqs=True, asr=self.asr, chneut=self.chneut, dipdip=self.dipdip,
                                    verbose=self.verbose, mpi_procs=self.mpi_procs)

        # Insert results in grid.
        gspec = pn.GridSpec(sizing_mode='scale_width')
        gspec[0, :1] = sv.get_dataframe()
        gspec[1, :1] = sv.plot(**self.fig_kwargs)

        return gspec

    @param.depends('plot_check_asr_dipdip_btn.clicks')
    def plot_without_asr_dipdip(self):
        if self.plot_check_asr_dipdip_btn.clicks == 0: return

        # Insert results in grid.
        gspec = pn.GridSpec(sizing_mode='scale_width')

        asr_plotter = self.ddb.anacompare_asr(asr_list=(0, 2), chneut_list=(1, ), dipdip=1,
                                              lo_to_splitting=self.lo_to_splitting,
                                              nqsmall=self.nqsmall, ndivsm=self.ndivsm,
                                              dos_method=self.dos_method, ngqpt=None,
                                              verbose=self.verbose, mpi_procs=self.mpi_procs)
        gspec[0, :1] = asr_plotter.plot(**self.fig_kwargs)

        dipdip_plotter = self.ddb.anacompare_dipdip(chneut_list=(1,), asr=2, lo_to_splitting=self.lo_to_splitting,
                                                    nqsmall=self.nqsmall, ndivsm=self.ndivsm,
                                                    dos_method=self.dos_method, ngqpt=None,
                                                    verbose=self.verbose, mpi_procs=self.mpi_procs)
        gspec[1, :1] = dipdip_plotter.plot(**self.fig_kwargs)
        return gspec

    @param.depends('plot_ifc_btn.clicks')
    def plot_ifc(self):
        if self.plot_ifc_btn.clicks == 0: return

        ifc = self.ddb.anaget_ifc(asr=self.asr, chneut=self.chneut, dipdip=self.dipdip)

        # Insert results in grid.
        gspec = pn.GridSpec(sizing_mode='scale_width')
        gspec[0, :1] = ifc.plot_longitudinal_ifc(title="Longitudinal IFCs", show=False)
        gspec[1, :1] = ifc.plot_longitudinal_ifc_short_range(title="Longitudinal IFCs short range", show=False)
        gspec[2, :1] = ifc.plot_longitudinal_ifc_ewald(title="Longitudinal IFCs Ewald", show=False)

        return gspec

    def get_panel(self):
        """Return tabs with widgets to interact with the DDB file."""
        tabs = pn.Tabs(); app = tabs.append
        row = pn.Row(bkw.PreText(text=self.ddb.to_string(verbose=self.verbose), sizing_mode="scale_both"))
        app(("Summary", row))
        app(("Ph-bands", pn.Row(
            pn.Column("# PH-bands options",
                      *[self.param[k] for k in ("nqsmall", "ndivsm", "asr", "chneut", "dipdip",
                                                "lo_to_splitting", "dos_method")],
                      self.temp_range, self.plot_phbands_btn),
            self.plot_phbands_and_phdos)
        ))
        app(("BECs", pn.Row(
            pn.Column("# Born effective charges options",
                     *[self.param[k] for k in ("asr", "chneut", "dipdip", "gamma_ev")], self.get_epsinf_btn),
            self.get_epsinf)
        ))
        app(("eps0", pn.Row(
            pn.Column("# epsilon_0",
                      *[self.param[k] for k in ("asr", "chneut", "dipdip", "gamma_ev")],
                      self.w_range, self.plot_eps0w_btn),
            self.plot_eps0w)
        ))
        app(("Speed of sound", pn.Row(
            pn.Column("# Speed of sound options",
                      *[self.param[k] for k in ("asr", "chneut", "dipdip")],
                      self.plot_vsound_btn),
            self.plot_vsound)
        ))
        app(("Check ASR and DIPDIP", pn.Row(
            pn.Column("# Options",
                      *[self.param[k] for k in ("nqsmall", "ndivsm", "dos_method")],
                      self.plot_check_asr_dipdip_btn),
            self.plot_without_asr_dipdip)
        ))

        app(("IFCs", pn.Row(
            pn.Column("# Options",
                      *[self.param[k] for k in ("asr", "dipdip", "chneut")],
                      self.plot_ifc_btn),
            self.plot_ifc)
        ))

        app(("Global Opts", pn.Column("# Global Options", *[self.param[k] for k in ("units", "mpi_procs", "verbose")])))

        return tabs



#!/usr/bin/env python
r"""
Structural relaxation
=====================

This example shows how to analyze the results of a
structure relaxation run using the HIST.nc file.
"""
from abipy.abilab import abiopen
import abipy.data as abidata

# Open the HIST file.
# (alternatively one can use the shell and `abiopen.py OUT_HIST.nc -nb`
# to open the file in jupyter notebook.
hist = abiopen(abidata.ref_file("sic_relax_HIST.nc"))

# The structure at the end of the structural relaxation.
print(hist.final_structure)

# Plot the evolution of the lattice parameters, forces, etotal, ...
hist.plot(tight_layout=True)

# Plot the total energy at the different relaxation steps.
hist.plot_energies(tight_layout=True)

hist.close()



"""
Utilities and helper functions.
"""
from __future__ import division, print_function

import wx

__all__ = [

]

def getApplicationConfigDirectory():
    """Returns the directory for application configuration information."""
    app = getApplication()
    return app.getConfigDirectory()


def getApplicationConfiguration():
    """Returns the application configuration object."""
    app = getApplication()
    return app.getConfiguration()


def beautifySize(size):
    """Returns a string representing the size in bytes in a more human readable way."""
    if size / 1073741824:
        return "%(size)0.2f GBytes" % {'size': (float(size) / 1073741824.0)}
    elif size / 1048576:
        return "%(size)0.2f MBytes" % {'size': (float(size) / 1048576.0)}
    elif size / 1024:
        return "%(size)0.2f KBytes" % {'size': (float(size) / 1024.0)}
    else:
        return "%(size)s Bytes" % {'size': size}


def addHorizontalSpaceTool(toolbar, width):
    """Adds horizontal space in a portable manner."""
    space = wx.StaticText(toolbar, -1, "")
    space.SetSize((width, -1))
    toolbar.AddControl(space)


def addLineSeparator(toolbar, height):
    """Adds a line separator to a toolbar."""
    addHorizontalSpaceTool(toolbar, 3)
    line = wx.StaticLine(toolbar, -1, style=wx.LI_VERTICAL)
    line.SetSize((-1, height))
    toolbar.AddControl(line)
    addHorizontalSpaceTool(toolbar, 3)


def getColumnText(list, index, col):
    """Gets the text from the specified column entry in a list."""
    item = list.GetItem(index, col)
    return item.GetText()


def getSelected(list):
    """Gets the selected items from a list object."""
    selected = []
    item = -1
    while 1:
        item = list.GetNextItem(item, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)
        if item == -1:
            break
        selected.append(item)
    return selected


def setListColumnAlignment(list, col, align):
    """Sets the column alignment for a column in a list."""
    item_info = list.GetColumn(col)
    item_info.SetAlign(align)
    list.SetColumn(col, item_info)


def colorToTuple(color):
    """Coverts a color object to a tuple RGB tuple representing the color."""
    return (color.Red(), color.Green(), color.Blue())


def isStrOrUnicode(value):
    """Returns true if the value is either a string or a unicode type."""
    return isinstance(value, str) or isinstance(value, unicode)


def is_string(s):
    """True if s behaves like a string (duck typing test)."""
    try:
        dummy = s + " "
        return True
    except TypeError:
        return False


def straceback(color=None):
    """
    Returns a string with the traceback.

    Use ANSII color formatting for output in terminal if color is not None.
    """
    import traceback
    s = traceback.format_exc()

    if color is not None:
        try:
            from termcolor import colored
            return colored(s, color)
        except ImportError:
            return s
    else:
        return s



#!/usr/bin/env python
r"""
Multiple phonon bands
=====================

This example shows how to plot several phonon band structures on a grid.

We use two files produced by anaddb:

    trf2_5.out_PHBST.nc: phonon frequencies on a q-path in the BZ (used to plot the band dispersion)
    trf2_5.out_PHDOS.nc: phonon DOS compute with anaddb.

See also tutorial/lesson_rf2.html
"""
from abipy import abilab
import abipy.data as abidata

# To plot a grid with two band structures:
phbst_paths = 2 * [abidata.ref_file("trf2_5.out_PHBST.nc")]

plotter = abilab.PhononBandsPlotter()
plotter.add_phbands("AlAs", phbst_paths[0])
plotter.add_phbands("Same AlAs", phbst_paths[1])

#plotter.combiplot(title="CombiPlot in eV")
plotter.gridplot(units="eV", title="GridPlot in eV")
plotter.boxplot(units="cm-1", title="BoxPlot in cm-1")
plotter.combiboxplot(units="Ha", title="CombiboxPlot in Ha")

# To plot a grid with band structures + DOS, use the optional argument `phdos` of add_phbands
# The first subplot gets the band dispersion from phbst_paths[0] and the dos from phdos_paths[0]
phbst_paths = 3 * [abidata.ref_file("trf2_5.out_PHBST.nc")]
phdos_paths = 3 * [abidata.ref_file("trf2_5.out_PHDOS.nc")]

plotter = abilab.PhononBandsPlotter()
plotter.add_phbands("AlAs phbands + DOS", phbst_paths[0], phdos=phdos_paths[0])
plotter.add_phbands("Same-data", phbst_paths[1], phdos=phdos_paths[1])
plotter.add_phbands("Same-data2", phbst_paths[2], phdos=phdos_paths[2])

#plotter.combiplot(title="Bands + DOS in eV with combiplot")
plotter.gridplot(units="cm-1", tight_layout=True, title="Bands + DOS in cm-1 with gridplot")
